# This sample tests that instance methods, regardless of how they're
# defined or decorated, act like instance methods.

from typing import Any, Callable, ClassVar, Concatenate, Generic, ParamSpec, TypeVar

P = ParamSpec("P")
R = TypeVar("R")


def func1(self) -> None:
    print("func1", f"{self=}")


def deco1(x: Callable[P, R]) -> Callable[P, R]:
    return x


def deco2(
    func: Callable[P, Any],
) -> Callable[[Callable[..., Any]], Callable[Concatenate["ClassA", P], None]]:
    return lambda f: f  # type: ignore


class Deco3(Generic[P, R]):
    def __init__(self, func: Callable[P, R]):
        self.func = func

    def __call__(self, *args: P.args, **kwargs: P.kwargs) -> R:
        print("Deco3.__call__:", f"{self=}")
        return self.func(*args, **kwargs)


class Deco4:
    def __init__(self, func: Callable[..., Any]):
        self.func = func

    def __call__(self) -> None:
        print("Deco4.__call__:", f"{self=}")


class CallableA:
    def __call__(self) -> None:
        print("CallableA.__call__:", f"{self=}")


class DummyClass:
    def __init__(self, a: str, b: float) -> None:
        pass


def dummyFunc(a: str, b: float) -> None:
    pass


class ClassA:
    a: ClassVar[Callable[[Any], None]] = lambda self: None

    b1 = lambda self: None
    b2: ClassVar = lambda self: None

    c1 = func1
    c2: ClassVar = func1

    d1: CallableA = CallableA()
    d2: ClassVar[CallableA] = CallableA()

    e1 = deco1(func1)
    e2: ClassVar = deco1(func1)

    @deco1
    def f1(self) -> None:
        print("f1:", f"{self=}")

    @Deco3
    def g1(self) -> None:
        print("g1:", f"{self=}")

    @Deco4
    def h1(self) -> None:
        print("h1:", f"{self=}")

    @deco2(DummyClass)
    def i1(self, a: str, b: float) -> None:
        print("i1:", f"{self=}")

    @deco2(dummyFunc)
    def j1(self, a: str, b: float) -> None:
        print("j1:", f"{self=}")


a = ClassA()

a.a()

a.b1()
a.b2()

a.c1()
a.c2()

a.d1()
a.d2()

a.e1()
a.e2()

a.f1()

a.g1(a)

a.h1()

a.i1("", 0)

a.j1("", 0)



from _typeshed import Incomplete
from typing import TypeVar

from pygments.formatter import Formatter

_T = TypeVar("_T", str, bytes)

class NullFormatter(Formatter[_T]):
    name: str
    aliases: Incomplete
    filenames: Incomplete
    def format(self, tokensource, outfile) -> None: ...

class RawTokenFormatter(Formatter[bytes]):
    name: str
    aliases: Incomplete
    filenames: Incomplete
    unicodeoutput: bool
    encoding: str
    compress: Incomplete
    error_color: Incomplete
    def format(self, tokensource, outfile) -> None: ...

class TestcaseFormatter(Formatter[_T]):
    name: str
    aliases: Incomplete
    def format(self, tokensource, outfile) -> None: ...



# This sample verifies that the reportOptionalOperand diagnostic
# isn't generated when the RHS operand accepts None.

# pyright: reportIncompatibleMethodOverride=false

from typing import Optional


class Cmp:
    def __eq__(self, other: "Optional[Cmp]") -> bool: ...

    def __lt__(self, other: "Optional[Cmp]") -> bool: ...

    def __gt__(self, other: "Cmp") -> bool: ...


def valid(value: Optional[Cmp], needed: Cmp):
    x = value > needed
    y = value == needed

    # This should generate an error if reportOptionalOperand is enabled.
    z = value < needed



# This sample tests the type analyzer's type narrowing
# logic for literals.

from typing import Literal, LiteralString, TypeVar, Union


def func1(p1: Literal["a", "b", "c"]):
    if p1 != "b":
        if p1 == "c":
            reveal_type(p1, expected_text="Literal['c']")
            pass
        else:
            reveal_type(p1, expected_text="Literal['a']")

    if p1 != "a":
        reveal_type(p1, expected_text="Literal['c', 'b']")
    else:
        reveal_type(p1, expected_text="Literal['a']")


def func2(p1: Literal[1, 4, 7]):
    if p1 == 4 or p1 == 1:
        reveal_type(p1, expected_text="Literal[4, 1]")
    else:
        reveal_type(p1, expected_text="Literal[7]")


def func3(a: Union[int, None]):
    if a == 1 or a == 2:
        reveal_type(a, expected_text="Literal[1, 2]")


T = TypeVar("T", bound=Literal["a", "b"])


def func4(x: T) -> T:
    if x == "a":
        reveal_type(x, expected_text="Literal['a']")
        return x
    else:
        reveal_type(x, expected_text="Literal['b']")
        return x


S = TypeVar("S", Literal["a"], Literal["b"])


def func5(x: S) -> S:
    if x == "a":
        reveal_type(x, expected_text="Literal['a']")
        return x
    else:
        reveal_type(x, expected_text="Literal['b']")
        return x


def func6(x: LiteralString):
    if x == "a":
        reveal_type(x, expected_text="Literal['a']")
    else:
        reveal_type(x, expected_text="LiteralString")



# This sample tests type inference for a generator returned
# by an __await__ function.

from collections.abc import Awaitable
from asyncio import get_event_loop, sleep


class MyAwaitable(Awaitable):
    def __await__(self):
        yield from (sleep(0.1).__await__())


async def func1():
    x: None = await MyAwaitable()


loop = get_event_loop()
loop.run_until_complete(func1())



# This sample tests the reportPropertyTypeMismatch diagnostic rule.

from typing import Generic, TypeVar

_T = TypeVar("_T")


class ClassA(Generic[_T]):
    @property
    def prop_1(self) -> float | None:
        return 2

    @prop_1.setter
    def prop_1(self, value: int) -> None:
        pass

    @property
    def prop_2(self) -> int | None:
        return 2

    # This should generate an error because a float
    # is not assignable to an Optional[int].
    @prop_2.setter
    def prop_2(self, value: float) -> None:
        pass

    @property
    def prop_3(self) -> list[_T]:
        return []

    # This should generate an error because _T is
    # not assignable to List[_T].
    @prop_3.setter
    def prop_3(self, value: _T) -> None:
        pass



import sys
from _typeshed import ReadOnlyBuffer, StrOrBytesPath
from types import TracebackType
from typing import Final, TypeVar, final, overload, type_check_only
from typing_extensions import Self, TypeAlias

if sys.platform != "win32":
    _T = TypeVar("_T")
    _KeyType: TypeAlias = str | ReadOnlyBuffer
    _ValueType: TypeAlias = str | ReadOnlyBuffer

    class error(OSError): ...
    library: Final[str]

    # Actual typename dbm, not exposed by the implementation
    @final
    @type_check_only
    class _dbm:
        def close(self) -> None: ...
        if sys.version_info >= (3, 13):
            def clear(self) -> None: ...

        def __getitem__(self, item: _KeyType) -> bytes: ...
        def __setitem__(self, key: _KeyType, value: _ValueType) -> None: ...
        def __delitem__(self, key: _KeyType) -> None: ...
        def __len__(self) -> int: ...
        def __enter__(self) -> Self: ...
        def __exit__(
            self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: TracebackType | None
        ) -> None: ...
        @overload
        def get(self, k: _KeyType, /) -> bytes | None: ...
        @overload
        def get(self, k: _KeyType, default: _T, /) -> bytes | _T: ...
        def keys(self) -> list[bytes]: ...
        def setdefault(self, k: _KeyType, default: _ValueType = b"", /) -> bytes: ...
        # This isn't true, but the class can't be instantiated. See #13024
        __new__: None  # type: ignore[assignment]
        __init__: None  # type: ignore[assignment]

    if sys.version_info >= (3, 11):
        def open(filename: StrOrBytesPath, flags: str = "r", mode: int = 0o666, /) -> _dbm:
            """
            Return a database object.

            filename
              The filename to open.
            flags
              How to open the file.  "r" for reading, "w" for writing, etc.
            mode
              If creating a new file, the mode bits for the new file
              (e.g. os.O_RDWR).
            """
            ...
    else:
        def open(filename: str, flags: str = "r", mode: int = 0o666, /) -> _dbm:
            """
            Return a database object.

            filename
              The filename to open.
            flags
              How to open the file.  "r" for reading, "w" for writing, etc.
            mode
              If creating a new file, the mode bits for the new file
              (e.g. os.O_RDWR).
            """
            ...



# This sample tests that overload matching for partially-overlapping overload
# signatures considers the "expected type" when using bidirectional type
# inference.

from typing import Any, Generic, LiteralString, TypeVar, overload


T = TypeVar("T")


class A(Generic[T]):
    @overload
    def func1(self: "A[bool]", x: "A[bool]") -> list[LiteralString]: ...

    @overload
    def func1(self, x: "A[str]") -> list[str]: ...

    def func1(self, x: "A[Any]") -> list[str] | list[LiteralString]:
        return []


def func2(a: A[bool], b: A[str]):
    v1: list[LiteralString] = a.func1(a)

    # This should generate an error.
    v2: list[str] = a.func1(a)

    # This should generate an error.
    v3: list[LiteralString] = b.func1(b)
    v4: list[str] = b.func1(b)



# This sample tests the case where a TypeVarTuple is solved using
# a tuple with literal values.

# Enable experimental features to support Union[*Ts].
# pyright: enableExperimentalFeatures=true

from typing import Callable, Literal, TypeVarTuple, Union, Unpack

Ts = TypeVarTuple("Ts")


def func1(
    f: Callable[[Unpack[Ts]], None], vs: tuple[Unpack[Ts]]
) -> Union[Unpack[Ts]]: ...


def func2(f: Callable[[Literal[1, 2]], None], vs: tuple[Literal[1, 2]]):
    v1 = func1(f, vs)
    reveal_type(v1, expected_text="Literal[1, 2]")


def func3(f: Callable[[Literal[1, 2, 3]], None], vs: tuple[Literal[1, 2]]):
    v1 = func1(f, vs)
    reveal_type(v1, expected_text="Literal[1, 2]")


def func4(f: Callable[[int], None], vs: tuple[Literal[1, 2]]):
    v1 = func1(f, vs)
    reveal_type(v1, expected_text="int")


def func5(f: Callable[[Literal[1, 2]], None], vs: tuple[Literal[1, 2, 3]]):
    # This should result in an error.
    func1(f, vs)


def func6(f: Callable[[Literal[1, 2]], None], vs: tuple[int]):
    # This should result in an error.
    func1(f, vs)


def func7(f: Callable[[int, int, int], None], vs: tuple[int, ...]):
    # This should result in an error because of a size mismatch.
    func1(f, vs)


def func8(f: Callable[[Unpack[tuple[int, ...]]], None], vs: tuple[int]):
    v1 = func1(f, vs)
    reveal_type(v1, expected_text="int")



# This sample tests expected diagnostics for augmented assignment
# expressions.


def func1(values1: list[float] = [], values2: list[float] | None = None) -> None:
    values3 = None

    # This should generate an error
    values1 += values2

    if values2 is not None:
        values1 += values2

    # This should generate an error
    values1 -= values2

    # This should generate an error
    values1 += values3



# This sample tests incompatible method overrides for multiple inheritance.
# This functionality is controlled by the reportIncompatibleMethodOverride
# diagnostic rule.


from typing import Generic, Iterable, ParamSpec, TypeVar, overload


class A1:
    def func1(self, a: int) -> str: ...


class A2:
    def func1(self, a: int, b: int = 3) -> str: ...


# This should generate an error because func1 is incompatible.
class ASub(A1, A2): ...


class B1:
    def func1(self) -> int: ...


class B2:
    def func1(self) -> float: ...


class BSub(B1, B2): ...


class C1:
    def func1(self) -> float: ...


class C2:
    def func1(self) -> int: ...


# This should generate an error because func1 is incompatible.
class CSub(C1, C2): ...


class D1:
    def func1(self, a: int) -> None: ...


class D2:
    def func1(self, b: int) -> None: ...


# This should generate an error because func1 is incompatible.
class DSub(D1, D2): ...


_T_E = TypeVar("_T_E")


class E1(Generic[_T_E]):
    def func1(self, a: _T_E) -> None: ...


class E2(Generic[_T_E]):
    def func1(self, a: _T_E) -> None: ...


class ESub(E1[int], E2[int]): ...


_T_F = TypeVar("_T_F")


class F1(Generic[_T_F]):
    def do_stuff(self) -> Iterable[_T_F]: ...


class F2(F1[_T_F]):
    def do_stuff(self) -> Iterable[_T_F]: ...


class F3(F1[_T_F]): ...


class FSub1(F3[int], F2[int]):
    pass


class FSub2(F3[int], F1[int]):
    pass


class FSub3(F2[int], F1[int]):
    pass


_P = ParamSpec("_P")
_R = TypeVar("_R")


class G1(Generic[_P, _R]):
    def f(self, *args: _P.args, **kwargs: _P.kwargs) -> _R: ...

    def g(self) -> _R: ...


class G2(G1[_P, _R]):
    # This should generate an error because f is missing ParamSpec parameters.
    def f(self) -> _R: ...

    def g(self, *args: _P.args, **kwargs: _P.kwargs) -> _R: ...


class G3(G1[[], _R]):
    def f(self) -> _R: ...

    def g(self) -> _R: ...


class G4(G1[[int, int], str]):
    def f(self, a: int, b: int, /) -> str: ...

    def g(self) -> str: ...


class G5(G1[[], str]):
    # This should generate an error because the specialized
    # signature of f in the base class has no positional parameters.
    def f(self, a: int, b: int) -> str: ...

    def g(self) -> str: ...


class H1:
    @property
    def prop1(self) -> int:
        return 3

    @property
    def prop2(self) -> int:
        return 3

    @prop2.setter
    def prop2(self, val: int) -> None:
        pass

    @property
    def prop3(self) -> int:
        return 3

    @prop3.setter
    def prop3(self, val: int) -> None:
        pass


class H2:
    @property
    def prop1(self) -> str:
        return ""

    @property
    def prop2(self) -> int:
        return 3

    @property
    def prop3(self) -> int:
        return 3

    @prop3.setter
    def prop3(self, val: str) -> None:
        pass


# This should generate three errors: prop1, prop2 and prop3.
class H(H2, H1): ...


class I1:
    @overload
    def func1(self, x: int) -> int: ...

    @overload
    def func1(self, x: str) -> str: ...

    def func1(self, x: int | str) -> int | str:
        return x


class I2:
    @overload
    def func1(self, x: int) -> int: ...

    @overload
    def func1(self, x: str) -> str: ...

    def func1(self, x: int | str) -> int | str:
        return x


class I(I1, I2): ...



# This sample tests a case that involves an interaction between a class
# that uses auto-variance and a decorator that uses a generic type alias.

from typing import Any, Concatenate, Generic, ParamSpec, TypeAlias, Callable
from typing_extensions import TypeVar  # pyright: ignore[reportMissingModuleSource]

T = TypeVar("T", infer_variance=True)
P = ParamSpec("P")
R = TypeVar("R")
S = TypeVar("S", bound="A[Any]")

TA1: TypeAlias = Callable[Concatenate[S, P], R]


def deco(func: TA1[S, P, R], /) -> TA1[S, P, R]: ...


class A(Generic[T]):
    @deco
    def select_all(self, *args: object) -> list[Any]: ...



# This sample tests the verification that overload implementation signatures
# are a superset of their associated overload signatures.

from typing import (
    Any,
    Awaitable,
    Callable,
    Generic,
    Iterable,
    Literal,
    NoReturn,
    TypeVar,
    overload,
)

T = TypeVar("T")


@overload
def func1(a: int) -> str: ...


@overload
def func1(a: str) -> int: ...


# This should generate two errors:
# The first is because of an incompatibility with overload 1
# because the input parameter type is incompatible.
# This second is because of an incompatibility with overload 2
# because the return type is incompatible.
def func1(a: str) -> str:
    return a


@overload
def func2(a: int, b: str = ...) -> str: ...


@overload
def func2(a: None) -> str: ...


# This should generate an error because the parameter "b" is missing
# from the implementation but is required by overload 1.
def func2(a: int | None) -> str: ...


@overload
def func3(a: int, *, b: Literal["r"]) -> str: ...


@overload
def func3(a: int, *, b: Literal["b"]) -> bytes: ...


def func3(*args: Any, **kwargs: Any) -> Any: ...


@overload
def func4(a: None) -> None: ...


@overload
def func4(a: list[T]) -> T: ...


def func4(a: list[T] | None) -> T | None: ...


class ClassA:
    @overload
    def method4(self, a: None) -> None: ...

    @overload
    def method4(self, a: list[T]) -> T: ...

    def method4(self, a: list[T] | None) -> T | None: ...


@overload
def func5(a: list[T]) -> T: ...


@overload
def func5(a: None) -> None: ...


# This should generate an error because list is not compatible with dict.
def func5(a: dict[Any, Any] | None) -> Any | None: ...


@overload
def func6(foo: int, /) -> int: ...


@overload
def func6(bar: str, /) -> int: ...


def func6(p0: int | str, /) -> int:
    return 3


class ClassB(Generic[T]):
    @overload
    def method1(self: "ClassB[None]") -> None: ...

    @overload
    def method1(self, value: T) -> None: ...

    def method1(self, value: Any = None) -> None: ...


class ClassC: ...


class ClassD: ...


T_CD = TypeVar("T_CD", ClassC, ClassD)


@overload
def func7(cls: type[ClassC], var: int) -> ClassC: ...


@overload
def func7(cls: type[ClassD], var: str) -> ClassD: ...


def func7(cls: type[T_CD], var: int | str) -> T_CD:
    return cls()


T_str = TypeVar("T_str", bound=str)


@overload
def func8(foo: int) -> int: ...


@overload
def func8(foo: T_str) -> tuple[T_str]: ...


def func8(foo: T_str | int) -> tuple[T_str] | int: ...


class ClassE: ...


T_E = TypeVar("T_E", bound=ClassE)


@overload
def func9() -> None: ...


@overload
def func9(bar: T_E) -> T_E: ...


def func9(bar: T_E | None = None) -> T_E | None:
    raise NotImplementedError


T_int_str = TypeVar("T_int_str", int, str)


@overload
def func10(option: Literal["a"], var: str) -> str: ...


@overload
def func10(option: Literal["b"], var: int) -> str: ...


# This should generate an error.
def func10(option: Literal["a", "b"], var: T_int_str) -> T_int_str: ...


class ClassF: ...


T_F = TypeVar("T_F", bound=type[ClassF])


@overload
def func11(var: T_F) -> T_F: ...


@overload
def func11(var: int) -> int: ...


def func11(var: T_F | int) -> T_F | int: ...


T7 = TypeVar("T7")
T8 = TypeVar("T8")
T9 = TypeVar("T9")


@overload
def func12(
    func: Callable[[T7], T8], iterable: Iterable[T7], default_value: None = None, /
) -> Iterable[T8 | None]: ...


@overload
def func12(
    func: Callable[[T7], T8], iterable: Iterable[T7], /, default_value: T9
) -> Iterable[T8 | T9]: ...


def func12(
    func: Callable[[T7], T8],
    iterable: Iterable[T7],
    /,
    default_value: T9 = None,
) -> Iterable[T8 | T9]: ...


@overload
def func13(x: int) -> NoReturn: ...


@overload
def func13(x: str) -> str | NoReturn: ...


def func13(x: int | str) -> str: ...


class ClassG(Generic[T]): ...


@overload
def func14(target: Callable[..., Awaitable[T]]) -> ClassG[T]: ...


@overload
def func14(target: Callable[..., T]) -> ClassG[T]: ...


def func14(
    target: Callable[..., Awaitable[T]] | Callable[..., T],
) -> ClassG[T]: ...


@overload
def func15(client_id: str, client_secret: str, /) -> None: ...


@overload
def func15(client_id: str, client_secret: str) -> None: ...


# This should generate an error because some of the keyword arguments are not present.
def func15(*creds: str) -> None:
    pass



# This sample tests the case where a comprehension-scoped variable
# shadows a variable of the same name in an outer scope and is
# narrowed within the comprehension.

def func1(m: list[str | int]) -> None:
    print(
        [
            reveal_type(value, expected_text="str")
            for value in m
            if isinstance(value, str)
        ]
    )

    reveal_type(value, expected_text="() -> None")
    value()


def value() -> None:
    pass



from _typeshed import Incomplete

esc: str
codes: Incomplete
dark_colors: Incomplete
light_colors: Incomplete

def reset_color(): ...
def colorize(color_key, text): ...
def ansiformat(attr, text): ...



# This sample tests f-strings where expressions contain
# other string literals.

# pyright: strict

a1 = f"[{{name}}{'}' if True else ''}]"

b1 = f"{'''hello'''}"

c1 = f"""{'"'}"""

hello1 = 3
d1 = f"{f'{hello1}'}"

print(f"{'a' if 'b' != d1 else 'd'}")


a2 = rf"[{{name}}{'}' if True else ''}]"

b2 = rf"{'''hello'''}"

c2 = rf"""{'"'}"""

hello2 = 3
d2 = rf"{rf'{hello2}'}"

e1 = f""" {f''' {"".join(["this", "that"])}'''}"""



# This sample tests TypeIs when used with a Callable type.

# pyright: reportMissingModuleSource=false

from typing import Callable
from typing_extensions import TypeIs


def is_callable(obj: object, /) -> TypeIs[Callable[..., object]]: ...


def func1(x: type[int]):
    if is_callable(x):
        reveal_type(x, expected_text="type[int]")
    else:
        reveal_type(x, expected_text="Never")


def func2[T](x: type[T]):
    if is_callable(x):
        reveal_type(x, expected_text="type[T@func2]")
    else:
        reveal_type(x, expected_text="Never")


def func3[T](x: type[T] | T):
    if is_callable(x):
        reveal_type(x, expected_text="type[T@func3] | ((...) -> object)")
    else:
        reveal_type(x, expected_text="object*")


def func4[T](x: T) -> T:
    if not is_callable(x):
        reveal_type(x, expected_text="object*")
        raise ValueError()
    reveal_type(x, expected_text="(...) -> object")
    return x



# This sample verifies that a generic dataclass works.

from dataclasses import dataclass
from typing import Generic, TypeVar, Union

T = TypeVar("T")


@dataclass
class ABase(Generic[T]):
    value: Union[str, T]


reveal_type(ABase(""), expected_text="ABase[Unknown]")


class AChild(ABase[int]):
    pass


reveal_type(AChild(123), expected_text="AChild")


class B(Generic[T]):
    pass


@dataclass
class CBase(Generic[T]):
    x: B[T] = B[T]()


@dataclass
class CChild(CBase[T]):
    pass


c1 = CBase[int]()
reveal_type(c1, expected_text="CBase[int]")
reveal_type(c1.x, expected_text="B[int]")

c2 = CChild[int]()
reveal_type(c2, expected_text="CChild[int]")
reveal_type(c2.x, expected_text="B[int]")



# This sample tests the handling of protocols with properties and
# methods that make use of generics.

from typing import Callable, Protocol, TypeVar

T = TypeVar("T")


class Proto(Protocol):
    @property
    def f(self: T) -> T: ...

    def m(self, item: T, callback: Callable[[T], str]) -> str: ...


class Concrete:
    @property
    def f(self: T) -> T:
        return self

    def m(self, item: T, callback: Callable[[T], str]) -> str: ...


x: Proto = Concrete()



# This sample tests the handling of immediately-called lambdas.

# pyright: strict

from typing import Callable

func1: Callable[[int, int], int] = lambda x, y, /: (
    lambda x, y: max(
        reveal_type(x, expected_text="int"), reveal_type(y, expected_text="int")
    )
    // min(x, y)
)(abs(reveal_type(x, expected_text="int")), abs(reveal_type(y, expected_text="int")))

v1 = func1(-2, 4)
reveal_type(v1, expected_text="int")


v2 = (lambda a, b: a + b)(3, 4)
reveal_type(v2, expected_text="int")


v3 = (lambda a, b: a + b)("foo", "bar")
reveal_type(v3, expected_text="LiteralString")

v4 = (lambda a, b: a + b)("foo", (lambda c, d: c + d)("b", "ar"))
reveal_type(v4, expected_text="LiteralString")



# Tests the type checker's handling of global and nonlocal keywords.

global a

a = 3
f = 3

# This should generate an error because nonlocal bindings aren't
# allowed at the module level.
nonlocal b


def func1():
    global a


def func2():
    global c


def func3():
    a = 3
    # This should generate an error because a is assigned locally
    # before its name binding is declared.
    global a

    d = 3
    h = 3

    def func3_1():
        nonlocal d

        h = 5

        # This should generate an error because h is assigned
        # locally before its name binding is declared.
        nonlocal h

        global e

        # This should generate an error because f is not available
        # in a nonlocal scope.
        nonlocal f

        nonlocal g

    e = 4
    g = 10



# This sample tests the case where a dataclass uses a ClassVar that
# is also Final.

from dataclasses import dataclass
from typing import ClassVar, Final


@dataclass
class A:
    a: Final[int]
    b: Final[str] = ""
    c: ClassVar[Final[int]] = 0
    d: ClassVar[Final] = 0
    e: Final[ClassVar[int]] = 0


a = A(1)

# This should generate an error.
a.a = 0

# This should generate an error.
a.b = ""

# This should generate an error.
a.c = 0

# This should generate an error.
A.c = 0

# This should generate an error.
A.d = 0

# This should generate an error.
A.e = 0


@dataclass
class B:
    a: ClassVar[Final[int]] = 0
    b: int = 1



# This sample tests the case where a function accesses its own decorated
# form within a loop.

from contextlib import contextmanager


@contextmanager
def func1():
    yield

    for _ in ():
        with func1():
            return



# This sample tests the type: ignore for individual lines.

from typing import Dict


a: int = 3
b = len(a) # type: ignore

for for for # type: ignore

c: Dict[str, str] = {
    3: 3,
    'hello': 3,
    3.2: 2.4
} #type:ignore # something








# This sample tests the handling of tuple expressions within a subscript
# when used with type annotations.

a1: dict[(str, str)] = {"hi": "there"}

# This should generate an error because there are too many type arguments.
a2: dict[(str, str, str)] = {"hi": "there"}

b1: list[(int,)] = [3, 4, 5]



# This sample tests bidirectional type inference for | operators. This
# should apply only to TypedDict types.

from typing import Literal, TypeVar, Generic, Callable

T1 = TypeVar("T1")
T2 = TypeVar("T2")


class S(Generic[T1]):
    def __or__(self, other: "S[T2]") -> "S[T1 | T2]": ...


def to(x: Callable[..., T1]) -> "S[T1]": ...


x1 = to(int) | to(float)


def func1(f: set[Literal["A", "B"]]):
    v1: set[Literal["A", "B"]] = f | f

    v2 = " ".join({"A"} | {"B"})



# This sample tests the case where an inheritance chain of
# dataclasses use generic types.

from dataclasses import dataclass
from typing import Generic, TypeVar

Key0 = TypeVar("Key0")
Key1 = TypeVar("Key1")
Key2 = TypeVar("Key2")
Value = TypeVar("Value")


@dataclass
class MapTreeLeaf(Generic[Key0, Value]):
    key: Key0
    value: Value


@dataclass
class MapTreeNode(MapTreeLeaf[Key1, Value]):
    pass


class Foo(Generic[Key2, Value]):
    def add(self, key: Key2, value: Value):
        return MapTreeNode(key=key, value=value)

    def test1(self, a: Key2, b: Value):
        v1 = self.add(a, b)
        reveal_type(v1, expected_text="MapTreeNode[Key2@Foo, Value@Foo]")
        reveal_type(v1.key, expected_text="Key2@Foo")
        reveal_type(v1.value, expected_text="Value@Foo")



# This sample tests protocol matching for a protocol and an implementation
# that use a mixture of class-scoped and function-scoped TypeVars.

from typing import Generic, Protocol, TypeVar

T1 = TypeVar("T1", covariant=True)
T2 = TypeVar("T2")


class ProtoA(Protocol[T1]):
    def method1(self, __key: str, __default: T2) -> "T1 | T2": ...


T3 = TypeVar("T3", covariant=True)
T4 = TypeVar("T4")


class A(Generic[T3]):
    def method1(self, key: str, default: T4) -> "T3 | T4":
        raise NotImplementedError


a1: A[str] = A()


def func1(storage: ProtoA[str]): ...


v1: ProtoA[str] = a1
func1(a1)


def func2(storage: ProtoA[int]): ...


# This should generate an error.
v2: ProtoA[int] = a1

# This should generate an error.
func2(a1)



# This sample tests for the check of a member access through a generic class
# when the type of the attribute is generic (and therefore its type is
# ambiguous).

from typing import Generic, TypeVar

T = TypeVar("T")


class ClassA(Generic[T]):
    x: T
    y: int

    def __init__(self, label: T | None = None) -> None: ...


ClassA[int].y = 1
ClassA[int].y
del ClassA[int].y

ClassA.y = 1
ClassA.y
del ClassA.y

# This should generate an error because x is generic.
ClassA[int].x = 1

# This should generate an error because x is generic.
ClassA[int].x

# This should generate an error because x is generic.
del ClassA[int].x

# This should generate an error because x is generic.
ClassA.x = 1

# This should generate an error because x is generic.
ClassA.x

# This should generate an error because x is generic.
del ClassA.x


class ClassB(ClassA[T]):
    pass


# This should generate an error because x is generic.
ClassB[int].x = 1

# This should generate an error because x is generic.
ClassB[int].x

# This should generate an error because x is generic.
del ClassB[int].x

# This should generate an error because x is generic.
ClassB.x = 1

# This should generate an error because x is generic.
ClassB.x

# This should generate an error because x is generic.
del ClassB.x


class ClassC(ClassA[int]):
    pass


ClassC.x = 1
ClassC.x
del ClassC.x
ClassC.x
del ClassC.x



# This sample tests the case where a lambda's type is determined using
# bidirectional type inference and one or more of the parameters
# corresponds to a generic type.

from typing import Callable, TypeVar, Generic, Any

T = TypeVar("T")
MsgT = TypeVar("MsgT", bound="Msg[Any]")


class Msg(Generic[T]):
    body: T


class Request: ...


def check(func: "Callable[[MsgT, int], object]") -> MsgT: ...


notification: Msg[Request] = check(lambda msg, foo: (msg.body, foo))



# This sample tests that bidirectional type inference works when
# assigning to a class-scoped variable that is annotated with a
# descriptor. The setter type should not be used in this case.


from typing import Callable, Generic, TypeVar


T = TypeVar("T")


class Desc1(Generic[T]):
    def __get__(
        self, instance: object | None, owner: type | None = None
    ) -> list[T]: ...

    def __set__(self, instance: object, value: list[T]) -> None: ...


def func1(factory: Callable[[], list[T]]) -> Desc1[T]: ...


class ClassA:
    not_working: Desc1[int] = func1(list)



from typing import Literal, cast, override
from abc import ABC, abstractmethod


class Foo:
    @property
    def deps(self) -> int: ...

class Bar(Foo): 
    deps = '' # reportIncompatibleMethodOverride & reportAssignmentType
              

class Baz:
    a: int | None = None

  
class Qux(Baz):
    a = 1

class A:
    @property
    def foo(Self) -> int: ...

class B(A):
    @property
    @override
    def foo(Self) -> str: ... # reportIncompatibleMethodOverride


class C:
    a = 1

class D(C):
    a: Literal[1] = 1 # reportIncompatibleUnannotatedOverride (because invariant)

class E(C):
    a = "" # reportIncompatibleUnannotatedOverride

class F:
    a: str | None = ""


class G(F):
    a = cast(str | None, "")


class H(G): 
    a = ""

class I(ABC):
    @property
    @abstractmethod
    def foo(self) -> int: ...

class J(I):
    foo = 1  # reportIncompatibleMethodOverride & reportAssignmentType


class K(J):
    foo = 2



# This sample tests the handling of binary operators when used with
# generic types.

from typing import TypeVar

_TInt = TypeVar("_TInt", bound=int)


def func1(n: _TInt) -> _TInt:
    x = n + 1
    reveal_type(x, expected_text="int")

    # This should generate an error.
    return x


_TIntOrStr = TypeVar("_TIntOrStr", int, str)


def func2(n: _TIntOrStr) -> _TIntOrStr:
    x = n + n
    reveal_type(x, expected_text="int* | str*")

    return x



# This sample tests whether metaclasses that support operator
# magic methods work correctly.

import ctypes

v1 = ctypes.POINTER(ctypes.c_bool) * 3
reveal_type(v1, expected_text="type[Array[_Pointer[c_bool]]]")

v2 = 3 * ctypes.POINTER(ctypes.c_bool)
reveal_type(v2, expected_text="type[Array[_Pointer[c_bool]]]")



# This sample tests issubclass calls that always evaluate to true.


def func1(p1: type[int], p2: type[int] | type[str]):
    a = issubclass(p2, str)

    b = issubclass(p2, (int, float))

    # This should generate an error because this is always true.
    c = issubclass(p2, (float, dict, int, str))

    d = issubclass(p1, float)

    e = issubclass(p2, (float, dict, int))

    # This should generate an error because this is always true.
    f = issubclass(p1, int)

    # This should not generate an error because it's within an assert.
    assert issubclass(p1, int)



# This sample validates that *P.args and **P.kwargs can be used as a
# tuple and dict, respectively.

# pyright: strict

from collections.abc import Callable
from typing import Any
from typing_extensions import ParamSpec  # pyright: ignore[reportMissingModuleSource]

P = ParamSpec("P")


def func1(func: Callable[P, object], *args: P.args, **kwargs: P.kwargs) -> str:
    arg_reprs = [repr(arg) for arg in args]
    arg_reprs.extend(k + "=" + repr(v) for k, v in kwargs.items())

    return func.__name__ + "(" + ", ".join(arg_reprs) + ")"


def func2(*values: object, sep: str | None = ..., end: str | None = ...) -> None: ...


func1(func2)


def func3(a: int, b: int): ...


def func4(*args: Any, **kwargs: Any):
    func1(func3, *args, **kwargs)



# This sample tests the handling of Callable fields within a
# dataclass definition.

# pyright: strict

from dataclasses import dataclass
from typing import Any, Callable, TypeVar

CallableT = TypeVar("CallableT", bound=Callable[..., Any])


def decorate(arg: CallableT) -> CallableT:
    return arg


def f(s: str) -> int:
    return int(s)


@dataclass
class C:
    str_to_int: Callable[[str], int] = f


c = C()


reveal_type(c.str_to_int, expected_text="(str) -> int")

c.str_to_int = decorate(f)



# This sample tests the handling of generic protocols or protocols
# with generic methods.

from typing import Protocol, Sequence, TypeVar

A = TypeVar("A")


class HasAdd1(Protocol[A]):
    def __add__(self: A, other: A) -> A: ...


T1 = TypeVar("T1", bound=HasAdd1)


def merge_element_lists1(a: Sequence[T1], b: Sequence[T1]) -> Sequence[T1]:
    retval: Sequence[T1] = []
    for a_elem in a:
        for b_elem in b:
            retval.append(a_elem + b_elem)
    return retval


# This is similar to HasAdd1 except that the class isn't generic.
class HasAdd2(Protocol):
    def __add__(self: A, other: A) -> A: ...


T2 = TypeVar("T2", bound=HasAdd2)


def merge_element_lists2(a: Sequence[T2], b: Sequence[T2]) -> Sequence[T2]:
    retval: Sequence[T2] = []
    for a_elem in a:
        for b_elem in b:
            retval.append(a_elem + b_elem)
    return retval



# This sample tests type compatibility between closed TypedDicts and
# dict and MutableMapping types.

from typing import MutableMapping, NotRequired, TypedDict
from typing_extensions import ReadOnly  # pyright: ignore[reportMissingModuleSource]


class IntDict1(TypedDict, extra_items=int):
    pass


class IntDictWithNum(IntDict1):
    num: NotRequired[int]


def func1(x: IntDict1) -> None:
    v: dict[str, int] = x
    v.clear()


def func2(x: dict[str, int]):
    # This should generate an error.
    not_required_num: IntDictWithNum = x


def func3(p1: IntDictWithNum, p2: dict[str, int]):
    d1: dict[str, int] = p1
    m1: MutableMapping[str, int] = p1
    func1(p1)

    # This should generate an error.
    d2: IntDictWithNum = p2


class IntDict2(TypedDict, extra_items=int):
    num: int


def func4(p1: IntDict2):
    # This should generate an error.
    d1: dict[str, int] = p1

    # This should generate an error.
    m1: MutableMapping[str, int] = p1

    # This should generate an error.
    func1(p1)


class IntDict3(TypedDict, extra_items=int):
    num: NotRequired[ReadOnly[int]]


def func5(p1: IntDict3):
    # This should generate an error.
    d1: dict[str, int] = p1

    # This should generate an error.
    m1: MutableMapping[str, int] = p1

    # This should generate an error.
    func1(p1)



# This sample tests the case where a LiteralString is used as the bound
# of a TypeVar.

from typing import Generic, LiteralString, TypeVar

T = TypeVar("T")
T_LS = TypeVar("T_LS", bound=LiteralString)


class ClassA(Generic[T]):
    def __init__(self, val: T) -> None: ...


def func1(x: T) -> ClassA[T]:
    return ClassA(x)


def func2(x: T_LS | None, default: T_LS) -> ClassA[T_LS]:
    if x is None:
        x = default

    reveal_type(x, expected_text="T_LS@func2")
    out = func1(x)
    reveal_type(out, expected_text="ClassA[T_LS@func2]")
    return out



# This sample verifies that the inferred return type
# of an async function is wrapped in a Coroutine.

import asyncio
from typing import Any, Coroutine


async def inspector(cr: Coroutine[Any, Any, Any]):
    return await cr


async def inner(sleep: int, message: str) -> str:
    await asyncio.sleep(sleep)
    print(message)
    return message


async def outer():
    await inspector(inner(1, "test"))


async def recursive1():
    await recursive1()



# This sample verifies that "Generic" is tagged as
# invalid in contexts outside of a class definition.

from typing import Generic, TypeVar


_T1 = TypeVar("_T1")


class ClassA(Generic[_T1]):
    pass


# This should generate an error.
def func1(a: _T1) -> Generic[_T1]: ...


# This should generate an error.
def func2(p1: Generic[_T1]) -> _T1: ...


TA1 = Generic


# This should generate an error.
def func3(a: _T1) -> TA1[_T1]: ...


class ClassB(TA1[_T1]): ...



# This sample tests that multi-exception lists are parsed correctly
# based on PEP 758 in Python 3.14.

def func1():
    try:
        pass
    # This should generate an error for Python 3.13 or earlier.
    except ZeroDivisionError, TypeError:
        raise

def func2():
    try:
        pass
    # This should generate an error because an "as" clause always requires parens.
    except ZeroDivisionError, TypeError as e:
        raise



# This sample tests that forward references to type aliases work.

from typing import Any, Union


class Base:
    @staticmethod
    def create(data: dict[str, Any]) -> "Mix":
        return A()


class A(Base):
    pass


class B(Base):
    pass


Mix = Union[A, B]


class S:
    @staticmethod
    def create(data: dict[str, Any]) -> "Mix":
        return A()



# This sample tests the functools.partial support for keyword parameters.

from functools import partial
from typing import Protocol


def func1(a: int, b: int, x: str) -> str:
    return x


class Proto1(Protocol):
    def __call__(self, x: str) -> str: ...


func2: Proto1 = partial(func1, 3, 4, x="a")
func2()
func2(x="b")


class Proto2(Protocol):
    def __call__(self, b: int) -> str: ...


func3: Proto2 = partial(func1, 3, b=3, x="a")
func3()
func3(x="b")
func3(b=3)
func3(x="b", b=3)
func3(b=3, x="b")



# This sample tests the case where return type inference encounters
# recursion.


class Grammar:
    @staticmethod
    def A():
        return Grammar.B

    @staticmethod
    def B():
        return Grammar.C

    @staticmethod
    def C():
        return Grammar.D

    @staticmethod
    def D():
        return Grammar.E

    @staticmethod
    def E():
        return Grammar.F

    @staticmethod
    def F():
        return Grammar.G

    @staticmethod
    def G():
        return Grammar.H

    @staticmethod
    def H():
        return Grammar.I

    @staticmethod
    def I():
        return Grammar.J

    @staticmethod
    def J():
        return Grammar.K

    @staticmethod
    def K():
        return Grammar.L

    @staticmethod
    def L():
        return Grammar.B


async def func1(a):
    if a == 0:
        return
    r = await func1(a - 1)
    return r



# This sample tests type inference for a lambda that has no inference
# context but has a default argument value.

lambda1 = lambda x="": x
reveal_type(lambda1, expected_text='(x: str = "") -> str')

lambda2 = lambda x=None: x
reveal_type(lambda2, expected_text="(x: Unknown | None = None) -> (Unknown | None)")



from __future__ import annotations

import json
from pathlib import Path
from typing import TYPE_CHECKING, ClassVar, Dict, TypedDict, Union, cast, final

from rich.highlighter import ReprHighlighter
from rich.text import Text
from textual import work
from textual.app import App, ComposeResult
from textual.color import Color
from textual.screen import Screen
from textual.widgets import Footer, Header, Label, Tabs, Tree
from typing_extensions import override

if TYPE_CHECKING:
    from collections.abc import Mapping

    from textual.widgets.tree import TreeNode

highlighter = ReprHighlighter()

# here we do find the project root dynamically.
_project_root = Path(__file__).parent

while all(entry.name != "pyproject.toml" for entry in _project_root.iterdir()):
    _project_root = _project_root.parent

LOCFILES_BASE = _project_root / "packages" / "pyright-internal" / "src" / "localization"

ALL_LANGUAGES = [t for x in LOCFILES_BASE.iterdir() if (t := x.stem[12:])]


class _LocMsgComment(TypedDict):
    """Commented localization message, only exists in 'en-us' language."""

    message: str
    """Original message in English."""
    comment: str | list[str]
    """
    Information used internally by microsoft's pylance team to tell their translators not to
    translate certain words. Use your judgement as to whether they should be followed. see the
    [localization docs](https://docs.basedpyright.com/dev/development/localization/#general-guidelines)
    for more information
    """


LocMessages = Dict[str, Union[str, _LocMsgComment]]


def read_locfile(language: str = "en-us") -> dict[str, LocMessages]:
    with (LOCFILES_BASE / f"package.nls.{language}.json").open(encoding="utf8") as f:
        return cast(dict[str, LocMessages], json.load(f))


LOCDATA_EN_US = read_locfile()


def diff_keys(orig: Mapping[str, object], comp: Mapping[str, object]):
    msgs: list[str] = []
    orig_set = set(orig.keys())
    comp_set = set(comp.keys())
    if orig_set != comp_set:
        msgs.append("Compared to the original:")
        if missing := orig_set - comp_set:
            msgs.append("these keys are missing: " + ", ".join(missing) + ".")
        if unused := comp_set - orig_set:
            msgs.append("these keys are unused: " + ", ".join(unused) + ".")
    else:
        msgs.append("This object has the same keys as the original.")
    return "\n".join(msgs)


@final
class LocDataTree(Tree[str]):
    _locnode: ClassVar[dict[str, TreeNode[str]]] = {}
    temp_comp = None
    lang: str = "en-us"

    @override
    def compose(self) -> ComposeResult:
        self.show_root = False
        yield from super().compose()

    @staticmethod
    def add_msgline(node: TreeNode[str], key: str, val: str):
        label = Text.assemble(Text.from_markup(f"[b]{key}[/b] = "), highlighter(repr(val)))
        node.data = key
        node.set_label(label)

    def add_node(self, name: str, data: LocMessages, node: TreeNode[str] | None = None) -> None:
        if name in self._locnode:
            node = self._locnode[name]
            node.remove_children()
        if node is None:
            node = self.root.add(name, name)
            self._locnode[name] = node
        node.set_label(Text(f"{{}} {name}"))
        for key, value in data.items():
            new_node = node.add_leaf("")
            if isinstance(value, str):
                self.add_msgline(new_node, key, value)
            else:
                self.add_msgline(new_node, key, value["message"])

    def load_data(self, lang: str = "en-us"):
        self.lang = lang
        data = read_locfile(lang)
        for dom, dat in data.items():
            self.add_node(dom, dat)

    def on_tree_node_selected(self, event: Tree.NodeSelected[str]) -> None:
        if self.lang == "en-us":
            return
        selected = event.node
        if selected.data is None or selected == self.temp_comp:
            return
        parent = selected.parent
        if not parent or parent.data is None:
            return
        if self.temp_comp is not None:
            self.temp_comp.remove()
        self.temp_comp = parent.add_leaf("", after=selected)
        msg = LOCDATA_EN_US[parent.data][selected.data]
        if isinstance(msg, str):
            self.add_msgline(self.temp_comp, f"{'':>{len(selected.data) - 7}}\\[en-us]", msg)
        else:
            self.add_msgline(
                self.temp_comp, f"{'':>{len(selected.data) - 7}}\\[en-us]", msg["message"]
            )


@final
class MsgDiffReport(Screen[None]):
    BINDINGS: ClassVar = [("c", "dismiss()")]
    lang = "en-us"

    @override
    def compose(self) -> ComposeResult:
        self.styles.align = "center", "middle"
        self.styles.background = Color(0, 0, 0, 0)
        yield (report := Label())
        report.styles.width = "70%"
        report.styles.height = "70%"
        report.styles.border = "solid", Color.parse("gray")
        report.border_title = "Message Keys Difference Report (Press 'c' to close this report)"
        self.compare(report)

    def compare(self, report: Label):
        if self.lang == "en-us":
            report.update("'en-us' is already the original file.")
            return
        data = read_locfile(self.lang)
        msgs: list[str] = []
        for origdom, origmsgs in LOCDATA_EN_US.items():
            msgs += ["", f"[{origdom}]", diff_keys(origmsgs, data.get(origdom, {}))]
        report.update("\n".join(msgs))


@final
class HelperTUI(App[None]):
    BINDINGS: ClassVar = [
        ("q", "quit()", "Quit program"),
        ("d", "toggle_dark", "Toggle dark mode"),
        ("c", "popup_keydiff", "Compare message keys differences"),
        ("r", "reload_tree", "Reload messages"),
    ]
    TITLE = "BasedPyright Localization Helper"

    @override
    def compose(self) -> ComposeResult:
        """Create child widgets for the app."""
        yield Header()
        yield Footer()
        yield (tabs := Tabs(*ALL_LANGUAGES))
        yield (loctree := LocDataTree("loctree"))
        loctree.load_data(tabs.active_tab.label_text if tabs.active_tab else "en-us")

    @override
    def action_toggle_dark(self) -> None:
        """An action to toggle dark mode."""
        self.theme = "textual-dark" if self.theme == "textual-light" else "textual-light"

    @work
    async def action_popup_keydiff(self) -> None:
        keydiff = MsgDiffReport()
        tabs = self.query_one(Tabs)
        keydiff.lang = tabs.active_tab.label_text if tabs.active_tab else "en-us"
        await self.push_screen_wait(keydiff)

    def action_reload_tree(self) -> None:
        tree = self.query_one(LocDataTree)
        if tree.temp_comp:
            tree.temp_comp.remove()
            tree.temp_comp = None
        tabs = self.query_one(Tabs)
        tree.load_data(tabs.active_tab.label_text if tabs.active_tab else "en-us")

    def on_tabs_tab_activated(self) -> None:
        self.action_reload_tree()


if __name__ == "__main__":
    app = HelperTUI()
    app.run()



# This sample tests the case where a generic class with multiple
# type parameters invokes its own constructor and uses its own
# type parameters to specialize the constructed type.

from typing import Generic, TypeVar

T = TypeVar("T")
U = TypeVar("U")


class Test1(Generic[T, U]):
    def __init__(self, t: T, u: U):
        pass

    def test1(self, ts: list[T], us: list[U]) -> None:
        # This should generate an error.
        x1: Test1[U, T] = Test1(us, ts)

        x2: Test1[list[U], list[T]] = Test1(us, ts)


class Test2(Generic[T, U]):
    def __init__(self):
        pass

    def test2(self) -> None:
        x1: Test2[U, T]
        x2: Test2[T, T]
        x3: Test2[T, U]

        x1 = Test2[U, T]()
        # This should generate an error.
        x2 = Test2[U, T]()
        # This should generate an error.
        x3 = Test2[U, T]()

        # This should generate an error.
        x1 = Test2[T, T]()
        x2 = Test2[T, T]()
        # This should generate an error.
        x3 = Test2[T, T]()

        # This should generate an error.
        x1 = Test2[T, U]()
        # This should generate an error.
        x2 = Test2[T, U]()
        x3 = Test2[T, U]()


class Test3(Generic[T, U]):
    def __init__(self, ts: list[T], us: list[U]):
        pass

    def test3(self, ts: list[T], us: list[U]) -> None:
        x1: Test3[U, T] = Test3(us, ts)

        # This should generate two errors.
        x2: Test3[list[U], list[T]] = Test3(us, ts)



# This sample tests type checking for match statements (as
# described in PEP 634) that contain mapping patterns.

from typing import Literal, TypedDict

from typing_extensions import NotRequired  # pyright: ignore[reportMissingModuleSource]


def test_unknown(value_to_match):
    match value_to_match:
        case {"hello": a1, **a2}:
            reveal_type(a1, expected_text="Unknown")
            reveal_type(a2, expected_text="dict[Unknown, Unknown]")
            reveal_type(value_to_match, expected_text="Unknown")


def test_object(value_to_match: object):
    match value_to_match:
        case {"hello": a1, **a2}:
            reveal_type(a1, expected_text="Unknown")
            reveal_type(a2, expected_text="dict[Unknown, Unknown]")
            reveal_type(value_to_match, expected_text="object")


def test_dict(value_to_match: dict[str | int, str | int]):
    match value_to_match:
        case {1: a1}:
            reveal_type(a1, expected_text="str | int")
            reveal_type(value_to_match, expected_text="dict[str | int, str | int]")

        case {"hi": b1, "hi2": b2, **b3}:
            reveal_type(b1, expected_text="str | int")
            reveal_type(b2, expected_text="str | int")
            reveal_type(b3, expected_text="dict[str | int, str | int]")
            reveal_type(value_to_match, expected_text="dict[str | int, str | int]")

        case {None: c1}:
            reveal_type(c1, expected_text="Never")
            reveal_type(value_to_match, expected_text="Never")


class Movie(TypedDict):
    title: str
    release_year: int
    gross_earnings: float


class MovieInfo:
    field_of_interest: Literal["release_year", "gross_earnings"]


def test_typed_dict(value_to_match: Movie):
    match value_to_match:
        case {"title": a1, "release_year": a2, **a3}:
            reveal_type(a1, expected_text="str")
            reveal_type(a2, expected_text="int")
            reveal_type(a3, expected_text="dict[str, object]")
            reveal_type(value_to_match, expected_text="Movie")

        case {3: b1, "title": b2}:
            reveal_type(b1, expected_text="Never")
            reveal_type(b2, expected_text="Never")
            reveal_type(value_to_match, expected_text="Never")

        case {"director": c1}:
            reveal_type(c1, expected_text="Never")
            reveal_type(value_to_match, expected_text="Never")

        case {MovieInfo.field_of_interest: d1}:
            reveal_type(d1, expected_text="int | float")
            reveal_type(value_to_match, expected_text="Movie")


def test_union1(value_to_match: dict[str | int, str | int] | Movie | str):
    match value_to_match:
        case {3: a1}:
            reveal_type(a1, expected_text="str | int")
            reveal_type(value_to_match, expected_text="dict[str | int, str | int]")

        case {"gross_earnings": b1}:
            reveal_type(b1, expected_text="str | int | float")
            reveal_type(
                value_to_match, expected_text="dict[str | int, str | int] | Movie"
            )


def test_union2(value_to_match: dict[int, int] | Movie | str):
    match value_to_match:
        case {**kw}:
            reveal_type(kw, expected_text="dict[int | str, int | object]")
            reveal_type(value_to_match, expected_text="dict[int, int] | Movie")

        case x:
            reveal_type(x, expected_text="str")


class IntValue(TypedDict):
    type: Literal["Int"]
    int_value: int


class StrValue(TypedDict):
    type: Literal["Str"]
    str_value: str


class ComplexValue(TypedDict):
    type: NotRequired[Literal["Complex"]]
    complex_value: complex


def test_negative_narrowing1(value: IntValue | StrValue | ComplexValue | int) -> None:
    match value:
        case {"type": "Int"}:
            reveal_type(value, expected_text="IntValue")
        case {"type": "Str" | "Complex"}:
            reveal_type(value, expected_text="StrValue | ComplexValue")
        case _:
            reveal_type(value, expected_text="ComplexValue | int")


def test_negative_narrowing2(value: StrValue | ComplexValue) -> None:
    if "type" not in value:
        raise

    match value:
        case {"type": "Str"}:
            reveal_type(value, expected_text="StrValue")
        case {"type": "Complex"}:
            reveal_type(value, expected_text="ComplexValue")
        case _:
            reveal_type(value, expected_text="Never")


class TD1(TypedDict):
    v1: NotRequired[int]
    v2: NotRequired[int]
    v3: NotRequired[int]


def test_not_required_narrowing(subj: TD1) -> None:
    match subj:
        case {"v1": _}:
            print(subj["v1"])

            # This should generate an error.
            print(subj["v2"])

        case {"v2": 1, "v3": 2}:
            # This should generate an error.
            print(subj["v1"])

            print(subj["v2"])
            print(subj["v3"])



# This sample tests a complex overload case that was causing a hang
# in pyright's logic.

from typing import Callable, overload


@overload
def func1[K, VI, VO](d: dict[K, VI], func: Callable[[VI], VO]) -> dict[K, VO]: ...


@overload
def func1[K, VI, VO](d: VI, func: Callable[[VI], VO]) -> VO: ...


def func1[K, VI, VO](
    d: dict[K, VI] | VI, func: Callable[[VI], VO]
) -> dict[K, VO] | VO: ...



# This sample tests the TypeGuard functionality
# that allows user-defined functions to perform
# conditional type narrowing.

# pyright: reportMissingModuleSource=false

import os
from typing import Any, Callable, TypeVar

from typing_extensions import TypeGuard  # pyright: ignore[reportMissingModuleSource]

_T = TypeVar("_T")


def is_two_element_tuple(a: tuple[_T, ...]) -> TypeGuard[tuple[_T, _T]]:
    return True


def func1(a: tuple[int, ...]):
    if is_two_element_tuple(a):
        reveal_type(a, expected_text="tuple[int, int]")
    else:
        reveal_type(a, expected_text="tuple[int, ...]")


def is_string_list(val: list[Any], allow_zero_entries: bool) -> TypeGuard[list[str]]:
    if allow_zero_entries and len(val) == 0:
        return True
    return all(isinstance(x, str) for x in val)


def func2(a: list[str | int]):
    if is_string_list(a, True):
        reveal_type(a, expected_text="list[str]")
    else:
        reveal_type(a, expected_text="list[str | int]")


# This should generate an error because TypeGuard
# has no type argument.
def bad1(a: int, b: object) -> TypeGuard:
    # This is a runtime use of TypeGuard and shouldn't generate an error.
    if b is TypeGuard:
        return True
    return True


# This should generate an error because TypeGuard
# has too many type arguments.
def bad2(a: int) -> TypeGuard[str, int]:
    return True


# This should generate an error because TypeGuard
# does not accept an ellipsis.
def bad3(a: int) -> TypeGuard[...]:
    return True


# This should generate an error because TypeGuard
# has does not accept a module.
def bad4(a: int) -> TypeGuard[os]:
    return True


def bad5(a: int) -> TypeGuard[int]:
    # This should generate an error because only
    # bool values can be returned.
    return 3


# This should generate an error because a type guard function must
# accept at least one parameter.
def bad6() -> TypeGuard[int]:
    return True


class ClassA:
    # This should generate an error because a type guard function must
    # accept at least one parameter.
    def method1(self) -> TypeGuard[int]:
        return True


class IsInt:
    def __call__(self, value: Any) -> TypeGuard[int]:
        return isinstance(value, int)


def func3(x: Any):
    i = IsInt()
    if i(x):
        reveal_type(x, expected_text="int")


def is_int(obj: type) -> TypeGuard[type[int]]: ...


def func4(typ: type[_T]) -> _T:
    if not is_int(typ):
        raise Exception("Unsupported type")

    return typ()


def takes_int_typeguard(f: Callable[[object], TypeGuard[int]]) -> None:
    pass


def int_typeguard(val: object) -> TypeGuard[int]:
    return isinstance(val, int)


def bool_typeguard(val: object) -> TypeGuard[bool]:
    return isinstance(val, bool)


def str_typeguard(val: object) -> TypeGuard[str]:
    return isinstance(val, str)


takes_int_typeguard(int_typeguard)
takes_int_typeguard(bool_typeguard)

# This should generate an error because TypeGuard is covariant.
takes_int_typeguard(str_typeguard)


v0 = is_int(int)
v1: bool = v0
v2: int = v0
v3 = v0 & v0



# This sample tests that generic type variables
# with constrained types properly generate errors. It tests
# both class-defined and function-defined type variables.

from typing import Generic, TypeVar


class Foo:
    var1: int

    def __call__(self, val: int):
        pass

    def do_stuff(self) -> int:
        return 0

    def __add__(self, val: "Foo") -> "Foo":
        return val


class Bar:
    var1: int
    var2: int

    def __call__(self, val: int):
        pass

    def do_stuff(self) -> float:
        return 0

    def do_other_stuff(self) -> float:
        return 0

    def __add__(self, val: "Bar") -> "Bar":
        return val


_T1 = TypeVar("_T1", Foo, Bar)
_T2 = TypeVar("_T2", Foo, Bar, str)


class ClassA(Generic[_T1, _T2]):
    async def func1(self, a: _T1) -> _T1:
        _ = a.var1

        # This should generate an error.
        _ = a.var2

        # This should generate an error.
        _ = a(3.3)

        # This should generate two errors.
        _ = a[0]

        # This should generate an error.
        _ = a + 1

        _ = a + a

        a += a

        # This should generate an error.
        _ = -a

        # This should generate an error.
        a += 3

        # This should generate an error.
        _ = await a

        # This should generate two errors.
        for _ in a:
            pass

        a.do_stuff()

        # This should generate an error.
        a.do_other_stuff()

        _ = a.__class__
        _ = a.__doc__

        return a

    async def func2(self, a: _T2, b: _T1) -> _T1:
        # This should generate two errors.
        _ = a.var2

        # This should generate an error.
        _ = a(3.3)

        # This should generate two errors.
        _ = a[0]

        # This should generate an error.
        _ = a + 1

        _ = a + a

        a += a

        # This should generate an error.
        _ = a + b

        # This should generate an error.
        _ = -a

        # This should generate an error.
        a += 3

        # This should generate an error.
        _ = await a

        # This should generate an error.
        for _ in a:
            pass

        # This should generate an error.
        a.do_other_stuff()

        _ = a.__class__
        _ = a.__doc__

        return b


_T3 = TypeVar("_T3", float, int, str)
_T4 = TypeVar("_T4", float, int)


def custom_add(a: _T3, b: _T4) -> float:
    if isinstance(a, str):
        return 0
    c = a + b
    reveal_type(c, expected_text="float* | int*")
    return c


class Thing1:
    def __add__(self, value: float) -> "Thing1": ...

    def __radd__(self, value: float) -> "Thing1": ...


class Thing2:
    def __add__(self, value: float) -> "Thing2": ...

    def __radd__(self, value: float) -> "Thing2": ...


TThing = TypeVar("TThing", Thing1, Thing2)


def func1(x: TThing) -> TThing:
    if isinstance(x, Thing1):
        return 2 + x
    else:
        assert isinstance(x, Thing2)
        return 3 + x


def func2(x: TThing) -> TThing:
    if isinstance(x, Thing1):
        return x + 2
    else:
        assert isinstance(x, Thing2)
        return x + 3



# This sample tests a complex combination of TypeVarTuple,
# unpacking, bidirectional type inference, and recursive calls.

from dataclasses import dataclass
from typing import Generic, TypeVar, TypeVarTuple, Callable

X = TypeVar("X")
Y = TypeVar("Y")
Z = TypeVar("Z")
Xs = TypeVarTuple("Xs")
Ys = TypeVarTuple("Ys")


def nil() -> tuple[()]:
    return ()


def cons(
    f: Callable[[X], Y],
    g: Callable[[*Xs], tuple[*Ys]],
) -> Callable[[X, *Xs], tuple[Y, *Ys]]:
    def wrapped(x: X, *xs: *Xs) -> tuple[Y, *Ys]:
        y, ys = f(x), g(*xs)
        return y, *ys

    return wrapped


def star(f: Callable[[X], Y]) -> Callable[[*tuple[X, ...]], tuple[Y, ...]]:
    def wrapped(*xs: X):
        if not xs:
            return nil()
        return cons(f, star(f))(*xs)

    return wrapped


@dataclass(frozen=True)
class Tree(Generic[X, Y]):
    left: X
    right: Y


def lift(
    f: Callable[[*Xs], tuple[*Ys]],
) -> Callable[[Tree[Z, tuple[*Xs]]], Tree[Z, tuple[*Ys]]]: ...


def test(
    f: Callable[[X], Y],
) -> Callable[[Tree[Z, tuple[X, ...]]], Tree[Z, tuple[Y, ...]]]:
    return lift(star(f))


def parallel(
    f: Callable[[X], Y],
    g: Callable[[*Xs], tuple[*Ys]],
) -> Callable[[X, *Xs], tuple[Y, *Ys]]:
    def wrapped(a: X, *bs: *Xs):
        return f(a), *g(*bs)

    return wrapped


def identity(x: X) -> X:
    return x


def parallel_identity(*xs: *Xs) -> tuple[*Xs]:
    return xs


Shape = TypeVarTuple("Shape")
DType = TypeVar("DType")


class NDArray(Generic[*Shape, DType]): ...


def insert(values: NDArray[*Shape, DType]) -> NDArray[int, *Shape, DType]: ...


def prepend(values: NDArray[*Shape, DType]) -> NDArray[int, *Shape, DType]:
    return insert(values)



# This sample tests ParamSpec (PEP 612) behavior.

from asyncio import Future
from typing import Awaitable, Callable, ParamSpec, TypeVar

P = ParamSpec("P")
R = TypeVar("R")


def awaitable_wrapper(a: Callable[P, R]) -> Callable[P, Awaitable[R]]:
    def foo_internal(*args: P.args, **kwargs: P.kwargs) -> Awaitable[R]:
        ft: "Future[R]" = Future()
        ft.set_result(a(*args, **kwargs))
        return ft

    return foo_internal


@awaitable_wrapper
def bar(a: int, b: str) -> float:
    return 2.3


async def bbb() -> float:
    return await bar(2, "3")



# This sample tests the handling of a union that includes both
# T and a generic class parameterized by T. This case is indeterminate
# according to PEP 484, but pyright has code in place to find the
# "least complex" answer.

from typing import Any, Generic, TypeVar, Union

T1 = TypeVar("T1")


class Wrapper(Generic[T1]): ...


def ensure_wrapped(item: Union[T1, Wrapper[T1]]) -> Wrapper[T1]: ...


def some_func(x: Wrapper[T1]) -> Wrapper[T1]:
    return ensure_wrapped(x)


def func1a(value: list[Union[T1, list[T1]]]) -> T1: ...


def func2a(value: list[Union[float, list[float]]]):
    x = func1a(value)
    reveal_type(x, expected_text="float")


def func3a(value: list[Union[str, list[float]]]):
    # This should generate an error
    func1a(value)


def func4a(value: list[Union[float, str, list[Union[float, str]]]]):
    x = func1a(value)
    reveal_type(x, expected_text="float | str")


def func1b(value: list[Union[int, list[T1]]]) -> T1: ...


def func2b(value: list[Union[int, list[float]]]):
    x = func1b(value)
    reveal_type(x, expected_text="float")


def func3b(value: list[Union[str, list[float]]]):
    # This should generate an error
    func1b(value)


def ensure_list(value: Union[T1, list[T1]]) -> list[T1]: ...


def func4(
    v1: list, v2: list[Any], v3: list[None], v4: Any, v5: int, v6: T1, v7: list[T1]
) -> T1:
    reveal_type(ensure_list(v1), expected_text="list[Unknown]")
    reveal_type(ensure_list(v2), expected_text="list[Any]")
    reveal_type(ensure_list(v3), expected_text="list[None]")
    reveal_type(ensure_list(v4), expected_text="list[Any]")
    reveal_type(ensure_list(v5), expected_text="list[int]")
    reveal_type(ensure_list(v6), expected_text="list[T1@func4]")
    reveal_type(ensure_list(v7), expected_text="list[T1@func4]")

    return v6



# This sample tests the case where protocol matching requires that the type
# parameter for the concrete class map to a union of types in the protocol.

from typing import Iterable, Protocol, TypeVar

K = TypeVar("K")
V = TypeVar("V")


class SpecialDict(Protocol[K, V]):
    def items(self) -> Iterable[tuple[K, V | int]]: ...

    def __getitem__(self, __key: K) -> V | int: ...

    def __setitem__(self, __key: K, __value: V | int) -> None: ...


def func1(k: K, v: V) -> SpecialDict[K, V]:
    x1: SpecialDict[K, V] = {k: v}
    x2: SpecialDict[K, V] = {k: v, k: 1}
    x3: SpecialDict[K, V] = {k: 0}
    return {}



# This sample tests protocol matching for modules when using
# a generic protocol class.

from typing import Protocol, TypeVar

from . import protocolModule3
from .protocolModule3 import Fn

X = TypeVar("X", covariant=True)
Z = TypeVar("Z")


class FnHandler(Protocol[X]):
    def __call__(self, x: Fn[X]) -> None: ...


class ModuleSpec(Protocol[Z]):
    x: FnHandler[Z]


m1: ModuleSpec[int] = protocolModule3
m1.x(lambda y: None)

# This should generate an error.
m2: ModuleSpec[str] = protocolModule3



# This sample tests the handling of the dataclass_transform mechanism
# when applied to a metaclass.

from typing import Any, TypeVar
from typing_extensions import (  # pyright: ignore[reportMissingModuleSource]
    dataclass_transform,
)

_T = TypeVar("_T")


class ModelField:
    def __init__(self, *, init: bool = True, default: Any | None = None) -> None: ...


def model_field(
    *, init: bool = True, default: Any | None = None, alias: str | None = None
) -> Any: ...


@dataclass_transform(
    kw_only_default=True,
    field_specifiers=(ModelField, model_field),
)
class ModelMeta(type):
    not_a_field: str


class ModelBase(metaclass=ModelMeta):
    def __init_subclass__(
        cls,
        *,
        frozen: bool = False,
        kw_only: bool = True,
        order: bool = True,
    ) -> None: ...


class Customer1(ModelBase, frozen=True):
    id: int = model_field()
    name: str = model_field()
    name2: str = model_field(alias="other_name", default="None")


# This should generate an error because a non-frozen class cannot
# derive from a frozen one.
class Customer1Subclass(Customer1, frozen=False):
    salary: float = model_field()


class Customer2(ModelBase, order=True):
    id: int
    name: str = model_field(default="None")


c1_1 = Customer1(id=3, name="Sue", other_name="Susan")

# This should generate an error because the class is frozen.
c1_1.id = 4

# This should generate an error because the class is kw_only.
c1_2 = Customer1(3, "Sue")

# This should generate an error because other_name is missing.
c1_3 = Customer1(id=3, name="John")

# This should generate an error because comparison methods are
# not synthesized.
v1 = c1_1 < c1_2

c2_1 = Customer2(id=0, name="John")

c2_2 = Customer2(id=1)

v2 = c2_1 < c2_2

# This should generate an error because Customer2 supports
# keyword-only parameters for its constructor.
c2_3 = Customer2(0, "John")


@dataclass_transform(frozen_default=True)
class ModelMetaFrozen(type):
    pass


class ModelBaseFrozen(metaclass=ModelMetaFrozen): ...


class Customer3(ModelBaseFrozen):
    id: int
    name: str


c3_1 = Customer3(id=2, name="hi")

# This should generate an error because Customer3 is frozen.
c3_1.id = 4



# This sample tests the code flow analysis used to determine
# whether it is safe to narrow the type of a captured variable.

from typing import NoReturn, Optional


def get_optional_int() -> Optional[int]: ...


v0 = get_optional_int()
if v0 is not None:
    # This should generate an error because v0 is
    # a global variable and could be reassigned
    # outside of this module.
    lambda: v0 + 5


def func0():
    v1 = get_optional_int()
    if v1 is not None:
        lambda: v1 + 5

    v2 = get_optional_int()
    if v2 is not None:
        # This should generate an error because v2
        # is reassigned after capture.
        lambda: v2 + 5
    v2 = None

    v3 = get_optional_int()
    if v3 is not None:
        lambda: v3 + 5
    else:
        v3 = None

    # This should generate an error because v4 is
    # not bound prior to the capture.
    lambda: v4 + 5
    v4 = get_optional_int()


def func1(v1: Optional[int]):
    if v1 is not None:
        lambda: v1 + 5


def func2(v1: Optional[int]):
    if v1 is not None:

        def func2_inner1():
            x = v1 + 5

            def func2_inner2():
                lambda: v1 + 5

            func2_inner2()

        func2_inner1()


def func3():
    v1: Optional[int] = 3
    lambda: v1 + 5


def func4():
    v1: Optional[int] = 3
    # This should generate an error because v1
    # is reassigned after capture.
    lambda: v1 + 5
    v1 = None


def func5():
    v1: Optional[int] = 3

    while True:
        lambda: v1 + 5


def func6():
    v1: Optional[int] = 3

    while True:
        if v1 is not None:
            # This should generate an error because
            # v1 is reassigned on a code path that is
            # reachable from the lambda.
            lambda: v1 + 5
        else:
            v1 = None


def func7():
    while True:
        # This should generate an error because v1 is
        # potentially unbound prior to capture.
        lambda: v1 + 5

        v1: Optional[int] = 3


def func8() -> NoReturn: ...


def func9(x: str | None):
    if not x:
        func8()

    def foo() -> str:
        return x.upper()

    return x.upper()


def func10(cond: bool, val: str):
    x: str | None = val if cond else None
    y: str | None = val if cond else None

    def inner1():
        nonlocal x
        x = None

    if x is not None and y is not None:

        def inner2():
            reveal_type(x, expected_text="str | None")
            reveal_type(y, expected_text="str")


def func11(foo: list[int] | None):
    if isinstance(foo, list):

        def inner() -> list[int]:
            return [x for x in foo]


def func12() -> None:
    counter = 0

    def inner() -> None:
        nonlocal counter
        reveal_type(counter, expected_text="int")
        counter += 1

    inner()



# This sample tests type checking for match statements (as
# described in PEP 634) that contain class patterns.

from typing import (
    Any,
    Generic,
    Literal,
    NamedTuple,
    Protocol,
    TypeVar,
    TypedDict,
    runtime_checkable,
)
from typing_extensions import (  # pyright: ignore[reportMissingModuleSource]
    LiteralString,
)
from dataclasses import dataclass, field

foo = 3

T = TypeVar("T")


class ClassA:
    __match_args__ = ("attr_a", "attr_b")
    attr_a: int
    attr_b: str


class ClassB(Generic[T]):
    __match_args__ = ("attr_a", "attr_b")
    attr_a: T
    attr_b: str


class ClassC: ...


class ClassD(ClassC): ...


def test_unknown(value_to_match):
    match value_to_match:
        case ClassA(attr_a=a2) as a1:
            reveal_type(a1, expected_text="ClassA")
            reveal_type(a2, expected_text="int")
            reveal_type(value_to_match, expected_text="ClassA")

        # This should generate an error because foo isn't instantiable.
        case foo() as a3:
            pass


def test_any(value_to_match: Any):
    match value_to_match:
        case list() as a1:
            reveal_type(a1, expected_text="list[Unknown]")
            reveal_type(value_to_match, expected_text="list[Unknown]")


def test_custom_type(value_to_match: ClassA | ClassB[int] | ClassB[str] | ClassC):
    match value_to_match:
        case int() as a1:
            reveal_type(a1, expected_text="Never")
            reveal_type(value_to_match, expected_text="Never")

        case ClassA(attr_a=a4, attr_b=a5) as a3:
            reveal_type(a3, expected_text="ClassA")
            reveal_type(a4, expected_text="int")
            reveal_type(a5, expected_text="str")
            reveal_type(value_to_match, expected_text="ClassA")
            reveal_type(value_to_match, expected_text="ClassA")

        case ClassB(a6, a7):
            reveal_type(a6, expected_text="int | str")
            reveal_type(a7, expected_text="str")
            reveal_type(value_to_match, expected_text="ClassB[int] | ClassB[str]")

        case ClassD() as a2:
            reveal_type(a2, expected_text="ClassD")
            reveal_type(value_to_match, expected_text="ClassD")

        case ClassC() as a8:
            reveal_type(a8, expected_text="ClassC")
            reveal_type(value_to_match, expected_text="ClassC")


def test_subclass(value_to_match: ClassD):
    match value_to_match:
        case ClassC() as a1:
            reveal_type(a1, expected_text="ClassD")

        case _ as a2:
            reveal_type(a2, expected_text="Never")


def test_literal(value_to_match: Literal[3]):
    match value_to_match:
        case float() as a2:
            reveal_type(a2, expected_text="Never")
            reveal_type(value_to_match, expected_text="Never")

        case str() as a3:
            reveal_type(a3, expected_text="Never")
            reveal_type(value_to_match, expected_text="Never")

        case int() as a1:
            reveal_type(a1, expected_text="Literal[3]")
            reveal_type(value_to_match, expected_text="Literal[3]")


def test_literal2(value_to_match: Literal[0, "1", b"2"]) -> None:
    match value_to_match:
        case float() as a2:
            reveal_type(a2, expected_text="Never")
            reveal_type(value_to_match, expected_text="Never")

        case str() as a3:
            reveal_type(a3, expected_text="Literal['1']")
            reveal_type(value_to_match, expected_text="Literal['1']")

        case int() as a1:
            reveal_type(a1, expected_text="Literal[0]")
            reveal_type(value_to_match, expected_text="Literal[0]")

        case x:
            reveal_type(x, expected_text='Literal[b"2"]')
            reveal_type(value_to_match, expected_text='Literal[b"2"]')


def test_literal_string(value_to_match: LiteralString) -> None:
    match value_to_match:
        case "a" as a1:
            reveal_type(value_to_match, expected_text="Literal['a']")
            reveal_type(a1, expected_text="Literal['a']")

        case str() as a2:
            reveal_type(value_to_match, expected_text="LiteralString")
            reveal_type(a2, expected_text="LiteralString")

        case a3:
            reveal_type(value_to_match, expected_text="Never")
            reveal_type(a3, expected_text="Never")


TFloat = TypeVar("TFloat", bound=float)


def test_bound_typevar(value_to_match: TFloat) -> TFloat:
    match value_to_match:
        case int() as a1:
            reveal_type(a1, expected_text="int*")
            reveal_type(value_to_match, expected_text="int*")

        case float() as a2:
            reveal_type(a2, expected_text="float*")
            reveal_type(value_to_match, expected_text="float*")

        case str() as a3:
            reveal_type(a3, expected_text="Never")
            reveal_type(value_to_match, expected_text="Never")

    return value_to_match


TInt = TypeVar("TInt", bound=int)


def test_union(
    value_to_match: TInt | Literal[3] | float | str,
) -> TInt | Literal[3] | float | str:
    match value_to_match:
        case int() as a1:
            reveal_type(a1, expected_text="int* | int")
            reveal_type(value_to_match, expected_text="int* | int")

        case float() as a2:
            reveal_type(a2, expected_text="float")
            reveal_type(value_to_match, expected_text="float")

        case str() as a3:
            reveal_type(a3, expected_text="str")
            reveal_type(value_to_match, expected_text="str")

    return value_to_match


T = TypeVar("T")


class Point(Generic[T]):
    __match_args__ = ("x", "y")
    x: T
    y: T


def func1(points: list[Point[float] | Point[complex]]):
    match points:
        case [] as a1:
            reveal_type(a1, expected_text="list[Point[float] | Point[complex]]")
            reveal_type(points, expected_text="list[Point[float] | Point[complex]]")

        case [Point(0, 0) as b1]:
            reveal_type(b1, expected_text="Point[float] | Point[complex]")
            reveal_type(points, expected_text="list[Point[float] | Point[complex]]")

        case [Point(c1, c2)]:
            reveal_type(c1, expected_text="float | complex")
            reveal_type(c2, expected_text="float | complex")
            reveal_type(points, expected_text="list[Point[float] | Point[complex]]")

        case [Point(0, d1), Point(0, d2)]:
            reveal_type(d1, expected_text="float | complex")
            reveal_type(d2, expected_text="float | complex")
            reveal_type(points, expected_text="list[Point[float] | Point[complex]]")

        case _ as e1:
            reveal_type(e1, expected_text="list[Point[float] | Point[complex]]")
            reveal_type(points, expected_text="list[Point[float] | Point[complex]]")


def func2(subj: object):
    match subj:
        case list() as a1:
            reveal_type(a1, expected_text="list[Unknown]")
            reveal_type(subj, expected_text="list[Unknown]")


def func3(subj: int | str | dict[str, str]):
    match subj:
        case int(x):
            reveal_type(x, expected_text="int")
            reveal_type(subj, expected_text="int")

        case str(x):
            reveal_type(x, expected_text="str")
            reveal_type(subj, expected_text="str")

        case dict(x):
            reveal_type(x, expected_text="dict[str, str]")
            reveal_type(subj, expected_text="dict[str, str]")


def func4(subj: object):
    match subj:
        case int(x):
            reveal_type(x, expected_text="int")
            reveal_type(subj, expected_text="int")

        case str(x):
            reveal_type(x, expected_text="str")
            reveal_type(subj, expected_text="str")


# Test the auto-generation of __match_args__ for dataclass.
@dataclass
class Dataclass1:
    val1: int
    val2: str = field(init=False)
    val3: complex


@dataclass
class Dataclass2:
    val1: int
    val2: str
    val3: float


def func5(subj: object):
    match subj:
        case Dataclass1(a, b):
            reveal_type(a, expected_text="int")
            reveal_type(b, expected_text="complex")
            reveal_type(subj, expected_text="Dataclass1")

        case Dataclass2(a, b, c):
            reveal_type(a, expected_text="int")
            reveal_type(b, expected_text="str")
            reveal_type(c, expected_text="float")
            reveal_type(subj, expected_text="Dataclass2")


@dataclass(match_args=False)
class DataclassNoMatchArgs:
    a: int
    b: int


def func5b(subj: object):
    match subj:
        # This should generate an error because match_args is False.
        case DataclassNoMatchArgs(1, 2):
            pass


# Test the auto-generation of __match_args__ for named tuples.
NT1 = NamedTuple("NT1", [("val1", int), ("val2", complex)])
NT2 = NamedTuple("NT2", [("val1", int), ("val2", str), ("val3", float)])


def func6(subj: object):
    match subj:
        case NT1(a, b):
            reveal_type(a, expected_text="int")
            reveal_type(b, expected_text="complex")
            reveal_type(subj, expected_text="NT1")

        case NT2(a, b, c):
            reveal_type(a, expected_text="int")
            reveal_type(b, expected_text="str")
            reveal_type(c, expected_text="float")
            reveal_type(subj, expected_text="NT2")


def func7(subj: object):
    match subj:
        case complex(real=a, imag=b):
            reveal_type(a, expected_text="float")
            reveal_type(b, expected_text="float")


T2 = TypeVar("T2")


class Parent(Generic[T]): ...


class Child1(Parent[T]): ...


class Child2(Parent[T], Generic[T, T2]): ...


def func8(subj: Parent[int]):
    match subj:
        case Child1() as a1:
            reveal_type(a1, expected_text="Child1[int]")
            reveal_type(subj, expected_text="Child1[int]")

        case Child2() as b1:
            reveal_type(b1, expected_text="Child2[int, Unknown]")
            reveal_type(subj, expected_text="Child2[int, Unknown]")


T3 = TypeVar("T3")


def func9(v: T3) -> T3 | None:
    match v:
        case str():
            reveal_type(v, expected_text="str*")
            return v

        case _:
            return None


T4 = TypeVar("T4", int, str)


def func10(v: T4) -> T4 | None:
    match v:
        case str():
            reveal_type(v, expected_text="str*")
            return v

        case int():
            reveal_type(v, expected_text="int*")
            return v

        case list():
            reveal_type(v, expected_text="Never")
            return v

        case _:
            return None


def func11(subj: Any):
    match subj:
        case Child1() as a1:
            reveal_type(a1, expected_text="Child1[Unknown]")
            reveal_type(subj, expected_text="Child1[Unknown]")

        case Child2() as b1:
            reveal_type(b1, expected_text="Child2[Unknown, Unknown]")
            reveal_type(subj, expected_text="Child2[Unknown, Unknown]")


class TD1(TypedDict):
    x: int


def func12(subj: int, flt_cls: type[float], union_val: float | int):
    match subj:
        # This should generate an error because int doesn't accept two arguments.
        case int(1, 2):
            pass

    match subj:
        # This should generate an error because float doesn't accept keyword arguments.
        case float(x=1):
            pass

    match subj:
        case flt_cls():
            pass

        # This should generate an error because it is a union.
        case union_val():
            pass

        # This should generate an error because it is a TypedDict.
        case TD1():
            pass


def func13(subj: tuple[Literal[0]]):
    match subj:
        case tuple((1,)) as a:
            reveal_type(subj, expected_text="Never")
            reveal_type(a, expected_text="Never")

        case tuple((0, 0)) as b:
            reveal_type(subj, expected_text="Never")
            reveal_type(b, expected_text="Never")

        case tuple((0,)) as c:
            reveal_type(subj, expected_text="tuple[Literal[0]]")
            reveal_type(c, expected_text="tuple[Literal[0]]")

        case d:
            reveal_type(subj, expected_text="Never")
            reveal_type(d, expected_text="Never")


class ClassE(Generic[T]):
    __match_args__ = ("x",)
    x: list[T]


class ClassF(ClassE[T]):
    pass


def func14(subj: ClassE[T]) -> T | None:
    match subj:
        case ClassF(a):
            reveal_type(subj, expected_text="ClassF[T@func14]")
            reveal_type(a, expected_text="list[T@func14]")
            return a[0]


class IntPair(tuple[int, int]):
    pass


def func15(x: IntPair | None) -> None:
    match x:
        case IntPair((y, z)):
            reveal_type(y, expected_text="int")
            reveal_type(z, expected_text="int")


def func16(x: str | float | bool | None):
    match x:
        case str(v) | bool(v) | float(v):
            reveal_type(v, expected_text="str | bool | float")
            reveal_type(x, expected_text="str | bool | float")
        case v:
            reveal_type(v, expected_text="int | None")
            reveal_type(x, expected_text="int | None")
    reveal_type(x, expected_text="str | bool | float | int | None")


def func17(x: str | float | bool | None):
    match x:
        case str() | float() | bool():
            reveal_type(x, expected_text="str | float | bool")
        case _:
            reveal_type(x, expected_text="int | None")
    reveal_type(x, expected_text="str | float | bool | int | None")


def func18(x: str | float | bool | None):
    match x:
        case str(v) | float(v) | bool(v):
            reveal_type(v, expected_text="str | float | bool")
            reveal_type(x, expected_text="str | float | bool")
        case _:
            reveal_type(x, expected_text="int | None")
    reveal_type(x, expected_text="str | float | bool | int | None")


T5 = TypeVar("T5", complex, str)


def func19(x: T5) -> T5:
    match x:
        case complex():
            return x
        case str():
            return x

    reveal_type(x, expected_text="float* | int*")
    return x


T6 = TypeVar("T6", bound=complex | str)


def func20(x: T6) -> T6:
    match x:
        case complex():
            return x
        case str():
            return x

    reveal_type(x, expected_text="float* | int*")
    return x


@runtime_checkable
class Proto1(Protocol):
    x: int


class Proto2(Protocol):
    x: int


def func21(subj: object):
    match subj:
        case Proto1():
            pass

        # This should generate an error because Proto2 isn't runtime checkable.
        case Proto2():
            pass


class Impl1:
    x: int


def func22(subj: Proto1 | int):
    match subj:
        case Proto1():
            reveal_type(subj, expected_text="Proto1")

        case _:
            reveal_type(subj, expected_text="int")



# This sample is taken from PEP 544. It verifies that
# the type checker allows instance variables that are initialized
# in a method to be counted toward conformance to a defined Protocol.

from typing import Protocol


class Template(Protocol):
    name: str  # This is a protocol member
    value: int = 0  # This one too (with default)

    def method(self) -> None:
        pass


class Concrete:
    def __init__(self, name: str, value: int) -> None:
        self.name = name
        self.value = value

    def method(self) -> None:
        return


var: Template = Concrete("value", 42)



# This sample tests a difficult set of circular dependencies
# between untyped variables.

# pyright: strict

from typing import Dict


class A:
    pass


class B(A):
    pass


def func1(v: A, s: Dict[B, A]) -> object:
    if not isinstance(v, B):
        return v
    u = s.get(v)
    while isinstance(u, B):
        v = u
        u = s.get(v)
    x = v if u is None else u
    return x



# This sample tests the type analyzer's ability to determine
# execution paths.


def func1(length: int) -> int:
    n = 0
    while True:
        if n >= length:
            return n
        n += 3
    else:
        # This should not be flagged as an error
        # because we should never get here.
        return "hello"

    # This should not be flagged as an error
    # because we should never get here.
    return "not_returned"


def func2() -> int:
    while None:
        # This should not be flagged as an error
        # because we should never get here.
        return "hello"
    else:
        # This should be an error because the return
        # type doesn't match.
        return "hello"

    # This should not be an error because we
    # should never get here.
    return "not_returned"


def func3() -> str:
    if True:
        return "hello"
    else:
        # This should not be flagged as an error
        # because we should never get here.
        return 21

    raise BaseException()

    # This should not be flagged as an error
    # because we should never get here.
    return 52


def func4(length: int) -> int:
    n = 0
    while True:
        if n >= length:
            return n
        n += 3
        break
    else:
        return "hello"

    # This should be flagged as an error because
    # the break in the while True loop means that
    # we might get here, and the return type does
    # not match.
    return "not_returned"



# This sample tests unpacking of tuples that contain PEP 646-style
# tuples with unknown length within them.


def suffix() -> tuple[int, str, *tuple[bool, ...]]:
    return 1, "a", True


def test_suffix():
    a1, a2, a3 = suffix()
    reveal_type(a1, expected_text="int")
    reveal_type(a2, expected_text="str")
    reveal_type(a3, expected_text="bool")

    *b1, b2, b3 = suffix()
    # This case is ambiguous.
    reveal_type(b1, expected_text="list[int]")
    reveal_type(b2, expected_text="str")
    reveal_type(b3, expected_text="bool")

    c1, *c2, c3 = suffix()
    # This case is ambiguous.
    reveal_type(c1, expected_text="int")
    reveal_type(c2, expected_text="list[str]")
    reveal_type(c3, expected_text="bool")

    d1, d2, *d3 = suffix()
    reveal_type(d1, expected_text="int")
    reveal_type(d2, expected_text="str")
    reveal_type(d3, expected_text="list[bool]")


def prefix() -> tuple[*tuple[int, ...], str, bool]:
    return 1, "a", True


def test_prefix():
    a1, a2, a3 = prefix()
    reveal_type(a1, expected_text="int")
    reveal_type(a2, expected_text="str")
    reveal_type(a3, expected_text="bool")

    *b1, b2, b3 = prefix()
    reveal_type(b1, expected_text="list[int]")
    reveal_type(b2, expected_text="str")
    reveal_type(b3, expected_text="bool")

    c1, *c2, c3 = prefix()
    # This case is ambiguous.
    reveal_type(c1, expected_text="int")
    reveal_type(c2, expected_text="list[str]")
    reveal_type(c3, expected_text="bool")

    d1, d2, *d3 = prefix()
    # This case is ambiguous.
    reveal_type(d1, expected_text="int")
    reveal_type(d2, expected_text="str")
    reveal_type(d3, expected_text="list[bool]")


def middle() -> tuple[int, *tuple[str, ...], bool]:
    return 1, "a", True


def test_middle():
    a1, a2, a3 = middle()
    reveal_type(a1, expected_text="int")
    reveal_type(a2, expected_text="str")
    reveal_type(a3, expected_text="bool")

    *b1, b2, b3 = middle()
    # This case is ambiguous.
    reveal_type(b1, expected_text="list[int]")
    reveal_type(b2, expected_text="str")
    reveal_type(b3, expected_text="bool")

    c1, *c2, c3 = middle()
    reveal_type(c1, expected_text="int")
    reveal_type(c2, expected_text="list[str]")
    reveal_type(c3, expected_text="bool")

    d1, d2, *d3 = middle()
    # This case is ambiguous.
    reveal_type(d1, expected_text="int")
    reveal_type(d2, expected_text="str")
    reveal_type(d3, expected_text="list[bool]")



# This sample tests the check for non-overlapping types compared
# with equals comparison.

from typing import Literal, TypeVar


OS = Literal["Linux", "Darwin", "Windows"]


def func1(os: OS, val: Literal[1, "linux"]):
    if os == "Linux":
        return True

    # This should generate an error because this expression will always
    # evaluate to False.
    if os == "darwin":
        return False

    # This should generate an error because this expression will always
    # evaluate to True.
    if os != val:
        return False

    # This should generate an error because this expression will always
    # evaluate to False.
    if val == 2:
        return False

    if val == 1:
        return True


class ClassA: ...


class ClassB: ...


_T1 = TypeVar("_T1")
_T2 = TypeVar("_T2", bound=ClassB)


def func2(a: ClassA, b: ClassB, c: _T1, d: _T2, e: ClassA | ClassB) -> None | _T1 | _T2:
    # This should generate an error because there is no overlap in types.
    if a == b:
        return

    # This should generate an error because there is no overlap in types.
    if a != b:
        return

    if a != c:
        return

    # This should generate an error because there is no overlap in types.
    if a != d:
        return

    if a == e:
        return

    if b == e:
        return


def func3(base: type) -> None:
    if base == ClassA:
        ...

    if ClassA == base:
        ...


def func4(val: str | None):
    # This should generate an error because there is no overlap in types.
    if val == 42:
        ...



# This sample tests the handling of generic type alias where a type
# argument is not provided.

# pyright: reportMissingTypeArgument=true

from typing import Any, Generic, TypeAlias, TypeVar

_T = TypeVar("_T")


class A(Generic[_T]): ...


# This should generate an error if reportMissingTypeArgument is enabled.
B: TypeAlias = A


v1: B = A()

# This should generate an error because B is already specialized.
v2: B[int] = A()

# This should generate an error if reportMissingTypeArgument is enabled.
v3: A = A()


C = A[str]


# This should generate an error because C is already specialized.
v4: C[int]


class D(Generic[_T]):
    def __getitem__(self, key: Any) -> int: ...


D_Alias = D[_T]

d: D_Alias[Any] = D()
item = d[0]

x: int = D_Alias[Any]()[0]

E: TypeAlias = _T

e1: E[int] = 3

# This should generate an error if reportMissingTypeArgument is enabled.
e2: E = 3



# This sample tests the "constrained TypeVar narrowing for return types"
# feature. When a declared return type of a function contains a constrained
# TypeVar and the return statement is found on a path that tests a variable
# that is typed as that TypeVar, we know that the code path is taken only
# in the case where constraint is satisfied.

from typing import AnyStr, Generic, ParamSpec, TypeVar
from typing_extensions import (  # pyright: ignore[reportMissingModuleSource]
    TypeVarTuple,
    Unpack,
)

_T1 = TypeVar("_T1", str, int)
_T2 = TypeVar("_T2")


class A: ...


class B: ...


class C: ...


_T3 = TypeVar("_T3", A, B, C)

_P = ParamSpec("_P")
_Ts = TypeVarTuple("_Ts")


def func1(val1: _T1) -> _T1:
    if isinstance(val1, str):
        return ""
    return 0


def func2(val1: _T1) -> list[_T1]:
    if isinstance(val1, str):
        return [""]
    return [0]


class Class1(Generic[_T1, _T2, _T3, _P, Unpack[_Ts]]):
    def meth1(
        self, val1: _T1, val2: _T2, val3: _T3, cond: bool
    ) -> list[_T1] | list[_T2] | list[_T3]:
        if cond:
            # This should generate an error.
            return [0]

        if cond or 3 > 2:
            if isinstance(val1, str):
                # This should generate an error.
                return [0]
            else:
                return [0]

        if cond or 3 > 2:
            if isinstance(val3, B):
                return [B()]
            else:
                # This should generate an error.
                return [C()]

        if cond or 3 > 2:
            if not isinstance(val3, B) and not isinstance(val3, C):
                return [A()]

        return [val1]

    def meth2(self, val1: _T1) -> _T1:
        val2 = val1

        while True:
            if isinstance(val2, str):
                return "hi"

            val2 = val2 = val1

            if isinstance(val2, int):
                return 0

    def meth3(self, val1: _T1, val2: _T3) -> _T1:
        if isinstance(val2, A):
            # This should generate an error.
            return 1

        if isinstance(val2, B):
            if isinstance(val1, str):
                return ""

        if isinstance(val1, int):
            if isinstance(val2, B):
                # This should generate an error.
                return ""

        raise BaseException()


def func3(s: AnyStr, y: AnyStr | None = None) -> AnyStr:
    if isinstance(s, str):
        if y is None:
            pass
        return ""
    else:
        raise NotImplementedError


def func4(t: _T3) -> _T3:
    match t:
        case A():
            return A()
        case B():
            return B()
        case C():
            return C()


def func5(t: _T3) -> _T3:
    if isinstance(t, A):
        return A()
    elif isinstance(t, B):
        return B()
    elif isinstance(t, C):
        return C()



# This sample tests type narrowing in a loop.

from typing_extensions import Self  # pyright: ignore[reportMissingModuleSource]
from collections.abc import Generator


class A:
    parent: Self | None


class B: ...


def foo(v: A | B | None) -> Generator[A, None, None]:
    reveal_type(v)
    if not isinstance(v, B):
        reveal_type(v, expected_text="A | None")
        while v is not None:
            reveal_type(v, expected_text="A")
            yield v
            v = v.parent
            reveal_type(v, expected_text="A | None")



# This sample tests the alternative syntax for unions as
# documented in PEP 604.

from typing import Callable, Generic, TypeVar, Union


def func1(a: int | str):
    if isinstance(a, int):
        return 1
    else:
        return 2


B = bytes | None | Callable[[], None]
A = int | str | B


def func2(a: A) -> B:
    if a == 3 or a is None:
        return b""
    elif not isinstance(a, (int, str, bytes)):
        a()


def func3(A: "int | str"):
    return 1


T = TypeVar("T")


def func4(a: str):
    def helper(value: T) -> T | None: ...

    class Baz(Generic[T]):
        qux: T | None

    reveal_type(helper(a), expected_text="str | None")
    reveal_type(Baz[str]().qux, expected_text="str | None")


T = TypeVar("T")
TT = TypeVar("TT", bound=type)


def decorator1(value: type[T]) -> type[T]: ...


def decorator2(value: TT) -> TT: ...


class ClassA:
    class ClassA_A:
        pass

    @decorator1
    class ClassA_B:
        pass

    @decorator2
    class ClassA_C:
        pass


a_or_str: "ClassA.ClassA_A | str"
b_or_str: "ClassA.ClassA_B | str"
b_or_str_Union: Union[ClassA.ClassA_B, str]
c_or_str: "ClassA.ClassA_C | str"

Alias1 = None | str
Alias2 = str | None

_T = TypeVar("_T")

Alias3 = _T | str
Alias4 = str | _T



# This sample tests the type checker's handling of chained
# comparison operators.

from datetime import datetime


def requires_bool(val: bool):
    pass


date1 = datetime.now()
date2 = datetime.now()
date3 = datetime.now()

foo1 = date1 < date2 <= date3
requires_bool(foo1)

int1 = 3
foo2 = 2 < int1 < 5
requires_bool(foo2)

# This should generate an error because
# int and datetime cannot be compared.
foo3 = date1 < date2 < 3

foo4 = (date1 < date2) < 3

foo5 = "1" == "1" in "1"

foo6 = "1" in "1" == "1"



# This sample tests the type analyzer's ability to flag attempts
# to instantiate abstract base classes.

from abc import ABC, abstractmethod


class AbstractClassA(ABC):
    @abstractmethod
    def foo1(self):
        pass

    @abstractmethod
    def foo2(self):
        pass

    def foo3(self):
        return 3

    @classmethod
    def foo4(cls):
        # This should not generate an error even though
        # it would appear to be attempting to instantiate
        # an abstract class. That's because we need to
        # assume that the caller is making this call on
        # a non-abstract subclass.
        return cls()


v1 = [subclass() for subclass in AbstractClassA.__subclasses__()]
reveal_type(v1, expected_text="list[AbstractClassA]")


# This should generate an error because AbstractFoo
# is an abstract class.
a = AbstractClassA()


class AbstractClassB(AbstractClassA):
    def foo1(self):
        pass


# This should generate an error because AbstractBar1
# is an abstract class.
b = AbstractClassB()


class AbstractClassC(AbstractClassB):
    def foo2(self):
        pass


# This should not generate an error because AbstractBar2
# overrides all of the abstract methods it inherits.
c = AbstractClassC()



# This sample tests the type checker's handling of
# augmented assignments (combining a binary operator
# with an assignment).


a = 1
b = 3.4

a += b
reveal_type(a, expected_text="float")

a -= b
reveal_type(a, expected_text="float")

a *= b
reveal_type(a, expected_text="float")

a /= b
reveal_type(a, expected_text="float")

a //= b
reveal_type(a, expected_text="float")

a %= b
reveal_type(a, expected_text="float")

a **= b
reveal_type(a, expected_text="Any")

a = 1

# This should generate an error because
# matrix multiply isn't supported by int.
a @= b

a |= b
a &= b
a ^= b
a <<= b
a >>= b


list1 = [1, 2, 3]
list1 += [4]

# This should generate an error
list1 += 4

# This should generate an error
list2 = [1]
list2 *= 4

# This should generate an error
list2 *= [4]


# Test __iadd__ override in list class, which accepts
# any iterator as an argument.
bar = ("d",)
foo = ["a", "b"]
foo += ["c"]
foo += bar



# This sample tests for isinstance calls that never evaluate to true.

from typing import final


class ABase: ...


@final
class AFinal(ABase): ...


class BBase: ...


@final
class BFinal(BBase): ...


def func1(a: AFinal, b: BFinal):
    # This should generate an error if reportUnnecessaryIsinstance is true.
    if isinstance(a, BBase):
        reveal_type(a)

    # This should generate an error if reportUnnecessaryIsinstance is true.
    if isinstance(a, BBase):
        reveal_type(a)


def func2(a: ABase, b: BBase):
    if isinstance(a, BBase):
        reveal_type(a)

    if isinstance(b, ABase):
        reveal_type(b)



# This sample tests the handling of class specialization expressions
# that provide signatures for ParamSpecs.

from typing import Any, Callable, Concatenate, Generic, ParamSpec, TypeVar


T = TypeVar("T")
P1 = ParamSpec("P1")
P2 = ParamSpec("P2")


class X(Generic[T, P1]):
    f: Callable[P1, int]
    x: T


def x1(x: X[int, P2]) -> str: ...


def x2(x: X[int, Concatenate[int, P2]]) -> str: ...


def X3(x: X[int, [int, bool]]) -> str: ...


def x4(x: X[int, ...]) -> str: ...


# This should generate an error because "int" can't be bound to a ParamSpec.
def x5(x: X[int, int]) -> str: ...


# This should generate an error.
def x6(x: X[..., ...]) -> str: ...


# This should generate an error.
def x7(x: X[[int], [int, int]]) -> str: ...


class Y(Generic[P2]):
    def __init__(self, cb: Callable[P2, Any]) -> None: ...

    def m1(self) -> X[int, Concatenate[float, P2]]: ...


y1 = Y(x4)
reveal_type(y1, expected_text="Y[(x: X[int, ...])]")

y2 = y1.m1()
reveal_type(y2, expected_text="X[int, (float, x: X[int, ...])]")


class Z(Generic[P1]):
    f: Callable[P1, int]


def z1(x: Z[[int, str, bool]]) -> str: ...


def z2(x: Z[int, str, bool]) -> str: ...


# This should generate an error.
def z3(x: Z[[int, [str], bool]]) -> str: ...


# This should generate an error.
def z4(x: Z[[[int, str, bool]]]) -> str: ...


# This should generate an error.
def z5(x: Z[[...]]) -> str: ...



# This sample tests pattern matching with tuple expansion when the number
# of expanded tuples grows very large.


class A1:
    pass


class A2:
    pass


class A3:
    pass


class A4:
    pass


class A5:
    pass


class A6:
    pass


class A7:
    pass


class A8:
    pass


class A9:
    pass


class A10:
    pass


class A11:
    pass


class A12:
    pass


class A13:
    pass


class A14:
    pass


class A15:
    pass


class A16:
    pass


class B1:
    pass


class B2:
    pass


class B3:
    pass


class B4:
    pass


class B5:
    pass


class B6:
    pass


class B7:
    pass


class B8:
    pass


class B9:
    pass


class B10:
    pass


class B11:
    pass


class B12:
    pass


class B13:
    pass


class B14:
    pass


class B15:
    pass


class B16:
    pass


type UA = (
    A1 | A2 | A3 | A4 | A5 | A6 | A7 | A8 | A9 | A10 | A11 | A12 | A13 | A14 | A15 | A16
)

type UB = (
    B1 | B2 | B3 | B4 | B5 | B6 | B7 | B8 | B9 | B10 | B11 | B12 | B13 | B14 | B15 | B16
)


def test(a: UA, b: UB) -> bool:
    t = a, b
    match t:
        case A1(), B1():
            return True
        case A2(), B2():
            return True
        case A3(), B3():
            return True
        case A4(), B4():
            reveal_type(t, expected_text="tuple[A4, B4]")
            return True
        case A5(), B5():
            return True
        case A6(), B6():
            reveal_type(t, expected_text="tuple[A6, B6]")
            return True
        case A7(), B7():
            reveal_type(t, expected_text="tuple[A7, B7]")
            return True
        case A8(), B8():
            reveal_type(t, expected_text="tuple[A8, B8]")
            return True
        case A9(), B9():
            # The type will become less precise in this case
            # because narrowing in the negative case needs
            # to fall back on less-precise types.
            reveal_type(t, expected_text="Sequence[A9 | B9]")
            return True
        case A10(), B10():
            return True
        case A11(), B11():
            return True
        case A12(), B12():
            return True
        case A13(), B13():
            return True
        case A14(), B14():
            return True
        case A15(), B15():
            return True
        case A16(), B16():
            return True
        case _:
            reveal_type(t, expected_text="Any")
            raise ValueError()



# This sample tests protocol matching for multiple protocols that refer
# to each other in a recursive fashion. In particular, this sample tests
# the case where a "cls" parameter is annotated with a protocol type.

from typing import Never, Self, TypeVar, Protocol

T_contra = TypeVar("T_contra", contravariant=True)
T = TypeVar("T")


class ProtoA(Protocol[T_contra, T]):
    def method1(self, value: T_contra) -> "ProtoA[T_contra, T]": ...

    @classmethod
    def method2(cls, value: T) -> T: ...


class ProtoB(Protocol[T_contra, T]):
    def method3(self) -> ProtoA[T_contra, T]: ...


class ImplA:
    def method1(self, value: int) -> Self: ...

    @classmethod
    def method2(cls, value: int) -> int: ...


class ImplB:
    def method3(self) -> ImplA: ...

    def method1(self, value: int) -> Self: ...

    @classmethod
    def method2(cls: type[ProtoB[Never, T]], value: list[T]) -> list[T]: ...


def func1(x: ProtoA[Never, T]) -> T: ...


v1 = func1(ImplB())
reveal_type(v1, expected_text="list[int]")



# This sample tests type narrowing of tuples based on len(x) test.

from typing import Callable, Literal, ParamSpec, TypeVar

P = ParamSpec("P")


def func1(val: tuple[int] | tuple[int, int] | tuple[str, str]):
    if len(val) == 1:
        reveal_type(val, expected_text="tuple[int]")
    else:
        reveal_type(val, expected_text="tuple[int, int] | tuple[str, str]")

    if len(val) != 2:
        reveal_type(val, expected_text="tuple[int]")
    else:
        reveal_type(val, expected_text="tuple[int, int] | tuple[str, str]")


def func2(val: tuple[int] | tuple[int, ...]):
    if len(val) == 1:
        reveal_type(val, expected_text="tuple[int]")
    else:
        reveal_type(val, expected_text="tuple[int, ...]")

    if len(val) != 2:
        reveal_type(val, expected_text="tuple[int] | tuple[int, ...]")
    else:
        reveal_type(val, expected_text="tuple[int, int]")


def func3(val: tuple[int] | tuple[()]):
    N = 0
    if len(val) == N:
        reveal_type(val, expected_text="tuple[()]")
    else:
        reveal_type(val, expected_text="tuple[int]")


_T1 = TypeVar("_T1", bound=tuple[int])
_T2 = TypeVar("_T2", bound=tuple[str, str])


def func4(val: _T1 | _T2) -> _T1 | _T2:
    if len(val) == 1:
        reveal_type(val, expected_text="_T1@func4")
    else:
        reveal_type(val, expected_text="_T2@func4")

    return val


def func5(
    val: (
        tuple[int, ...]
        | tuple[str]
        | tuple[str, str, str]
        | tuple[int, *tuple[str, ...], str]
        | tuple[int, *tuple[float, ...]]
    ),
    length: Literal[2],
):
    if len(val) == length:
        reveal_type(
            val, expected_text="tuple[int, int] | tuple[int, str] | tuple[int, float]"
        )
    else:
        reveal_type(
            val,
            expected_text="tuple[int, ...] | tuple[str] | tuple[str, str, str] | tuple[int, str, *tuple[str, ...], str] | tuple[int, *tuple[float, ...]]",
        )


def func10(t: tuple[()] | tuple[int] | tuple[int, int] | tuple[int, int, int]):
    if len(t) >= 2:
        reveal_type(t, expected_text="tuple[int, int] | tuple[int, int, int]")
    else:
        reveal_type(t, expected_text="tuple[()] | tuple[int]")


def func11(t: tuple[()] | tuple[int] | tuple[int, int] | tuple[int, int, int]):
    if len(t) > 1:
        reveal_type(t, expected_text="tuple[int, int] | tuple[int, int, int]")
    else:
        reveal_type(t, expected_text="tuple[()] | tuple[int]")


def func12(t: tuple[()] | tuple[int] | tuple[int, int]):
    if len(t) >= 0:
        reveal_type(t, expected_text="tuple[()] | tuple[int] | tuple[int, int]")
    else:
        reveal_type(t, expected_text="Never")


def func20(t: tuple[int, ...]):
    if len(t) >= 2:
        reveal_type(t, expected_text="tuple[int, int, *tuple[int, ...]]")
    else:
        reveal_type(t, expected_text="tuple[()] | tuple[int]")


def func21(t: tuple[int, ...]):
    if len(t) > 0:
        reveal_type(t, expected_text="tuple[int, *tuple[int, ...]]")
    else:
        reveal_type(t, expected_text="tuple[()]")


def func22(t: tuple[str, *tuple[int, ...], str]):
    if len(t) < 3:
        reveal_type(t, expected_text="tuple[str, str]")
    else:
        reveal_type(t, expected_text="tuple[str, int, *tuple[int, ...], str]")


def func23(t: tuple[str, *tuple[int, ...], str]):
    if len(t) <= 3:
        reveal_type(t, expected_text="tuple[str, str] | tuple[str, int, str]")
    else:
        reveal_type(t, expected_text="tuple[str, int, int, *tuple[int, ...], str]")


def func24(t: tuple[str, *tuple[int, ...], str]):
    if len(t) <= 34:
        reveal_type(t, expected_text="tuple[str, *tuple[int, ...], str]")
    else:
        reveal_type(t, expected_text="tuple[str, *tuple[int, ...], str]")


def func25(t: tuple[str, *tuple[int, ...], str]):
    if len(t) < 2:
        reveal_type(t, expected_text="Never")
    else:
        reveal_type(t, expected_text="tuple[str, *tuple[int, ...], str]")


def func26(fn: Callable[P, None]):
    def inner(*args: P.args, **kwargs: P.kwargs):
        if len(args) >= 0:
            reveal_type(args, expected_text="P@func26.args")
        else:
            reveal_type(args, expected_text="P@func26.args")
        return fn(*args, **kwargs)

    return inner


def func27(t: tuple[int, ...]):
    if len(t) == 0 or len(t) >= 2:
        reveal_type(t, expected_text="tuple[()] | tuple[int, int, *tuple[int, ...]]")
    else:
        reveal_type(t, expected_text="tuple[int]")


def func28(t: tuple[int, *tuple[int, ...]]):
    if len(t) == 1 or len(t) >= 3:
        reveal_type(
            t, expected_text="tuple[int] | tuple[int, int, int, *tuple[int, ...]]"
        )
    else:
        reveal_type(t, expected_text="tuple[int, int]")



# This sample tests scoping rules for variables that are declared within
# a list comprehension statement.

a: str = "hello"

# The statement "len(a)" should not generate an
# error because "a" is not yet bound to the local
# variable at the time it is executed. Instead, it
# has the str type of the "a" in the outer scope.
b = [a for a in [len(a)]]

# This assignment should succeed because "a" at this
# point should have the type of "a" in the outer scope,
# not the int type from the list comprehension.
c: str = a



# This sample tests the case where a variable with a declared type
# is assigned an unknown value or partially-unknown value.


def a_test(x1: int, x2: list):
    u = x1.upper()  # type: ignore
    reveal_type(u, expected_text="Unknown")

    # This should generate an error if reportUnknownVariableType is enabled.
    y: str = u
    reveal_type(y, expected_text="Unknown | str")

    # This should generate an error if reportUnknownVariableType is enabled.
    z: list[str] = x2
    reveal_type(z, expected_text="list[str]")


def b_test(x: int | str):
    u = x.upper()  # type: ignore
    reveal_type(u, expected_text="str | Unknown")

    # This should generate an error if reportUnknownVariableType is enabled.
    y: str = u
    reveal_type(y, expected_text="str | Unknown")



# This sample tests the case where a constrained TypeVar
# is used in a protocol with a contravariant TypeVar.

# This example is a stand-alone sample based on the following code.
# with open('read', 'rb') as fr, open('write', 'wb') as fw:
#     shutil.copyfileobj(fr, fw)

from typing import Any, AnyStr, Optional, Protocol, TypeVar, Union


class Array: ...


class MMap: ...


# Note that this union contains types that are not compatible
# with the type "bytes".
ReadableBuffer = Union[bytes, bytearray, memoryview, Array, MMap]

_T_contra = TypeVar("_T_contra", contravariant=True)
_T_co = TypeVar("_T_co", covariant=True)


class BufferedWriter:
    def write(self, __buffer: ReadableBuffer) -> int: ...


class SupportsWrite(Protocol[_T_contra]):
    def write(self, __s: _T_contra) -> Any: ...


class SupportsRead(Protocol[_T_co]):
    def read(self, __length: int = ...) -> _T_co: ...


class BufferedReader:
    def read(self, __size: Optional[int] = ...) -> bytes: ...


def copyfileobj(
    fsrc: SupportsRead[AnyStr], fdst: SupportsWrite[AnyStr], length: int = ...
) -> AnyStr: ...


def f(fr: BufferedReader, fw: BufferedWriter):
    x = copyfileobj(fr, fw)
    reveal_type(x, expected_text="bytes")



# This sample tests the case where an object is assigned to a
# callback protocol and the object's "__call__" has unannotated
# or "Any" parameter types.

from typing import Any, Protocol, TypeVar

InputT = TypeVar("InputT", contravariant=True)
OutputT = TypeVar("OutputT", covariant=True)


class MyCallable(Protocol[InputT, OutputT]):
    def __call__(self, inputs: InputT) -> OutputT: ...


class Class1:
    def __call__(self, inputs) -> int:
        return 5


g1: MyCallable[int, int] = Class1()


class Class2:
    def __call__(self, inputs: Any) -> int:
        return 5


g2: MyCallable[int, int] = Class2()



# This sample tests that the functools.partial special-case logic
# properly handles bidirectional type inference for argument evaluation.

from functools import partial


class BaseClass:
    pass


class SubClass(BaseClass):
    pass


def func_base(base: BaseClass):
    pass


def func_list(base: list[BaseClass]):
    pass


def func_set(base: set[BaseClass]):
    pass


sub = SubClass()

partial(func_base, sub)
partial(func_list, [sub])
partial(func_set, {sub})



# This sample tests the handling of calls with unpacked TypedDicts.


from typing_extensions import TypedDict  # pyright: ignore[reportMissingModuleSource]


class ClosedTD1(TypedDict, closed=True):
    arg1: str


class IntDict1(TypedDict, extra_items=int):
    arg1: str


td1 = ClosedTD1(arg1="hello")
td2 = IntDict1(arg1="hello", arg2=3)


def func1(arg1: str):
    pass


func1(**td1)

# This should arguably generate an error because there
# could be extra items, but we'll match mypy's behavior here.
func1(**td2)


def func2(arg1: str, **kwargs: str):
    pass


func2(**td1)

# This should result in an error.
func2(**td2)



# This sample verifies that the type checker properly handles
# lambdas with position-only and keyword-only markers.

from typing import Callable

foo1: Callable[[int], int] = lambda x, /: x + 1

# This should generate an error because there are too few
# parameters provided by the lambda.
foo2: Callable[[int, int], int] = lambda x, /: x + 1

# This should generate an error because there are too many
# parameters provided by the lambda.
foo3: Callable[[int, int], int] = lambda x, /, y, z: x + 1

# This should generate an error because there is no named
# parameter y.
foo4: Callable[[int, int], int] = lambda x, *, y: x + y + 1

# This should generate an error because there are too few
# parameters provided by the lambda.
foo5: Callable[[int, int, int], int] = lambda x, *, y: x + y + 1

# This should generate an error because there are too many
# parameters provided by the lambda.
foo6: Callable[[int], int] = lambda x, *, y: x + y + 1

foo7: Callable[[int, str], int] = lambda *args: 1

# This should generate an error because there are too many
# parameters provided by the lambda.
foo8: Callable[[int, str], int] = lambda a, b, c, *args: 1



# This sample tests the type checker's ability to handle
# custom subclasses of property.


from typing import Any, Callable


class custom_property1(property):
    pass


class Custom1(object):
    @custom_property1
    def x(self) -> int:
        return 3

    @custom_property1
    def y(self) -> float:
        return 3.5

    @y.setter
    def y(self, val: float):
        pass

    @y.deleter
    def y(self):
        pass


m1 = Custom1()

a1: int = m1.x

# This should generate an error because m.x is
# an int and cannot be assigned to str.
b1: str = m1.x

c1: float = m1.y

# This should generate an error because m.y is
# a float and cannot be assigned to int.
d1: int = m1.y

# This should generate an error because there
# is no setter for x.
m1.x = 4

m1.y = 4

# This should generate an error because there is
# no deleter for x.
del m1.x

del m1.y


class custom_property2(property):
    _custom_func: Callable[..., Any] | None

    def custom_function(self, _custom_func: Callable[..., Any]):
        self._custom_func = _custom_func
        return self


class Custom2(object):
    @custom_property2
    def x(self) -> int:
        return 3

    @custom_property2
    def y(self) -> float:
        return 3.5

    @y.setter
    def y(self, val: float):
        pass

    @y.deleter
    def y(self):
        pass

    @y.custom_function
    def y(self):
        pass


m2 = Custom2()

a2 = m2.y
reveal_type(a2, expected_text="float")

m2.y = 4
del m2.y



# This sample tests a loop that references instance variables.


class Results:
    zzz: int


class Foo:
    yyy: int

    def method1(self, results: list[Results]):
        abc = None
        for result in results:
            if abc is not None and abc.zzz < result.zzz:
                abc = result



# This sample tests nonlocal and global bindings
# with declared types.

foo: int = 23
baz: int = 23


def func():
    foo: str = "Hi"
    baz: str = "Hi"

    def func_1():
        global foo

        # This should generate an error because
        # the global "foo" is typed as a str.
        foo = "25"

        global bar

        bar: str = "Hi"

        nonlocal baz

        # This should generate an error because the
        # nonlocal "baz" is typed as str.
        baz = 25

    func_1()


func()

# This should generate an error because the
# type of "bar" is defined in func_1.
bar = 24



# This sample tests the reportUnusedVariable diagnostic check.


def func1(a: int):
    x = 4

    # This should generate an error if reportUnusedVariable is enabled.
    y = x

    _z = 4

    _ = 2

    __z__ = 5

    if x + 1:
        # This should generate an error if reportUnusedVariable is enabled.
        z = 3
    else:
        # This should generate an error if reportUnusedVariable is enabled.
        z = 5



# This sample tests the special-case handling of an overloaded __init__
# method where the "self" parameter is specialized to influence the
# type of the constructed object.

from typing import Any, Generic, Literal, Optional, Type, TypeVar, overload

_T = TypeVar("_T", bound=Optional[str])


class TextField(Generic[_T]):
    @overload
    def __init__(self: "TextField[str]", *, null: Literal[False] = ...) -> None: ...

    @overload
    def __init__(
        self: "TextField[Optional[str]]",
        *,
        null: Literal[True] = ...,
    ) -> None: ...

    @overload
    def __init__(self, *, null: bool = ...) -> None: ...

    def __init__(self, *, null: bool = ...) -> None: ...

    def __get__(self: "TextField[_T]", instance: Any, owner: Any) -> _T: ...


def foo(a: bool):
    reveal_type(TextField(), expected_text="TextField[str]")
    reveal_type(TextField(null=True), expected_text="TextField[str | None]")
    reveal_type(TextField(null=a), expected_text="TextField[Unknown]")


class Model: ...


_T1 = TypeVar("_T1", bound="Optional[Model]")
_T2 = TypeVar("_T2", bound="Optional[Model]")


class ForeignKey(Generic[_T1]):
    @overload
    def __init__(
        self: "ForeignKey[_T2]", to: Type[_T2], *, null: Literal[False] = ...
    ) -> None: ...

    @overload
    def __init__(
        self: "ForeignKey[Optional[_T2]]", to: Type[_T2], *, null: Literal[True]
    ) -> None: ...

    def __init__(self, to: Type[_T2], *, null: bool = False) -> None: ...


class Author(Model):
    pass


reveal_type(ForeignKey(Author, null=False), expected_text="ForeignKey[Author]")
reveal_type(ForeignKey(Author, null=True), expected_text="ForeignKey[Author | None]")


_T3 = TypeVar("_T3")
_T4 = TypeVar("_T4")
_S1 = TypeVar("_S1")
_S2 = TypeVar("_S2")


class Class1(Generic[_T3, _T4]):
    def __init__(self: "Class1[_S1, _S2]", value1: _S1, value2: _S2) -> None: ...


reveal_type(Class1(0, ""), expected_text="Class1[int, str]")


class Class2(Generic[_T3, _T4]):
    def __init__(self: "Class2[_S2, _S1]", value1: _S1, value2: _S2) -> None: ...


reveal_type(Class2(0, ""), expected_text="Class2[str, int]")


class Class3(Generic[_T3, _T4]):
    # This should generate an error because class-scoped TypeVars are not
    # allowed in the "self" type annotation for an __init__ method.
    def __init__(self: "Class3[_T3, _T4]", value1: _T3, value2: _T4) -> None: ...



# This sample tests that the tokenizer properly handles
# line feeds.

"""
This is a multi-line comment \
with escape characters.
"""

# This is a raw string with an escaped EOL.
foo = r"\
"

# The final token should be on line 14
bar = foo



# This sample tests the matching of nested callables that each use
# ParamSpec.

from typing import Callable, Generic, TypeVar
from typing_extensions import (  # pyright: ignore[reportMissingModuleSource]
    Concatenate,
    ParamSpec,
)

P = ParamSpec("P")
Q = ParamSpec("Q")

T = TypeVar("T")
U = TypeVar("U")


class ClassA(Generic[P, T, Q, U]): ...


def decorator1(
    func: Callable[Concatenate[Callable[P, T], Q], U],
) -> ClassA[P, T, Q, U]: ...


@decorator1
def func1(func: Callable[[int], float], a: str) -> bool: ...


reveal_type(func1, expected_text="ClassA[(int), float, (a: str), bool]")



# This sample tests the type checker's validation
# of the __init_subclass__ method described in
# PEP 487.

from datetime import datetime
from typing import Any, Optional, Type, TypedDict


class ClassA:
    def __init_subclass__(
        cls, *, param1: str, param2: float, param3: Optional[Any] = None
    ) -> None:
        super().__init_subclass__()


# This should generate two errors because param1 is
# the wrong type.
class ClassB(ClassA, param1=0, param2=4):
    pass


# This should generate two errors because param2 is missing.
class ClassC(ClassA, param1="0", param3=datetime.now()):
    pass


class ClassD(ClassA, param1="0", param2=5.0):
    pass


class ClassE:
    def __init_subclass__(cls, *, arg: int) -> None:
        func1(cls, arg)

    def __new__(cls) -> "ClassE":
        func1(cls, 9)
        return super().__new__(cls)


def func1(klass: Type[ClassE], arg: int):
    pass


class ClassF(ClassA, param1="hi", param2=3.4):
    def __init_subclass__(cls, param_alt1: int):
        super().__init_subclass__(param1="yo", param2=param_alt1)


def func2(cls):
    pass


class ClassG:
    __init_subclass__ = func2


class ClassH(ClassG):
    pass


# This should generate two errors because "a" is not present
# in the object.__init_subclass__ method.
class ClassI(a=3):
    a: int


class ClassJ:
    def __init_subclass__(cls, **kwargs: Any) -> None:
        super().__init_subclass__(**kwargs)
        cls.custom_attribute = 9


class ClassJChild(ClassJ):
    def __init__(self):
        reveal_type(self.custom_attribute, expected_text="int")



"""
This module provides access to the Windows registry API.

Functions:

CloseKey() - Closes a registry key.
ConnectRegistry() - Establishes a connection to a predefined registry handle
                    on another computer.
CreateKey() - Creates the specified key, or opens it if it already exists.
DeleteKey() - Deletes the specified key.
DeleteValue() - Removes a named value from the specified registry key.
DeleteTree() - Deletes the specified key and all its subkeys and values recursively.
EnumKey() - Enumerates subkeys of the specified open registry key.
EnumValue() - Enumerates values of the specified open registry key.
ExpandEnvironmentStrings() - Expand the env strings in a REG_EXPAND_SZ
                             string.
FlushKey() - Writes all the attributes of the specified key to the registry.
LoadKey() - Creates a subkey under HKEY_USER or HKEY_LOCAL_MACHINE and
            stores registration information from a specified file into that
            subkey.
OpenKey() - Opens the specified key.
OpenKeyEx() - Alias of OpenKey().
QueryValue() - Retrieves the value associated with the unnamed value for a
               specified key in the registry.
QueryValueEx() - Retrieves the type and data for a specified value name
                 associated with an open registry key.
QueryInfoKey() - Returns information about the specified key.
SaveKey() - Saves the specified key, and all its subkeys a file.
SetValue() - Associates a value with a specified key.
SetValueEx() - Stores data in the value field of an open registry key.

Special objects:

HKEYType -- type object for HKEY objects
error -- exception raised for Win32 errors

Integer constants:
Many constants are defined - see the documentation for each function
to see what constants are used, and where.
"""

import sys
from _typeshed import ReadableBuffer, Unused
from types import TracebackType
from typing import Any, Final, Literal, final, overload
from typing_extensions import Self, TypeAlias

if sys.platform == "win32":
    _KeyType: TypeAlias = HKEYType | int
    def CloseKey(hkey: _KeyType, /) -> None: ...
    def ConnectRegistry(computer_name: str | None, key: _KeyType, /) -> HKEYType: ...
    def CreateKey(key: _KeyType, sub_key: str | None, /) -> HKEYType: ...
    def CreateKeyEx(key: _KeyType, sub_key: str | None, reserved: int = 0, access: int = 131078) -> HKEYType: ...
    def DeleteKey(key: _KeyType, sub_key: str, /) -> None: ...
    def DeleteKeyEx(key: _KeyType, sub_key: str, access: int = 256, reserved: int = 0) -> None: ...
    def DeleteValue(key: _KeyType, value: str, /) -> None: ...
    def EnumKey(key: _KeyType, index: int, /) -> str: ...
    def EnumValue(key: _KeyType, index: int, /) -> tuple[str, Any, int]: ...
    def ExpandEnvironmentStrings(string: str, /) -> str: ...
    def FlushKey(key: _KeyType, /) -> None: ...
    def LoadKey(key: _KeyType, sub_key: str, file_name: str, /) -> None: ...
    def OpenKey(key: _KeyType, sub_key: str | None, reserved: int = 0, access: int = 131097) -> HKEYType: ...
    def OpenKeyEx(key: _KeyType, sub_key: str | None, reserved: int = 0, access: int = 131097) -> HKEYType: ...
    def QueryInfoKey(key: _KeyType, /) -> tuple[int, int, int]: ...
    def QueryValue(key: _KeyType, sub_key: str | None, /) -> str: ...
    def QueryValueEx(key: _KeyType, name: str, /) -> tuple[Any, int]: ...
    def SaveKey(key: _KeyType, file_name: str, /) -> None: ...
    def SetValue(key: _KeyType, sub_key: str | None, type: int, value: str, /) -> None: ...
    @overload  # type=REG_DWORD|REG_QWORD
    def SetValueEx(
        key: _KeyType, value_name: str | None, reserved: Unused, type: Literal[4, 5], value: int | None, /
    ) -> None: ...
    @overload  # type=REG_SZ|REG_EXPAND_SZ
    def SetValueEx(
        key: _KeyType, value_name: str | None, reserved: Unused, type: Literal[1, 2], value: str | None, /
    ) -> None: ...
    @overload  # type=REG_MULTI_SZ
    def SetValueEx(
        key: _KeyType, value_name: str | None, reserved: Unused, type: Literal[7], value: list[str] | None, /
    ) -> None: ...
    @overload  # type=REG_BINARY and everything else
    def SetValueEx(
        key: _KeyType,
        value_name: str | None,
        reserved: Unused,
        type: Literal[0, 3, 8, 9, 10, 11],
        value: ReadableBuffer | None,
        /,
    ) -> None: ...
    @overload  # Unknown or undocumented
    def SetValueEx(
        key: _KeyType,
        value_name: str | None,
        reserved: Unused,
        type: int,
        value: int | str | list[str] | ReadableBuffer | None,
        /,
    ) -> None: ...
    def DisableReflectionKey(key: _KeyType, /) -> None: ...
    def EnableReflectionKey(key: _KeyType, /) -> None: ...
    def QueryReflectionKey(key: _KeyType, /) -> bool: ...

    HKEY_CLASSES_ROOT: Final[int]
    HKEY_CURRENT_USER: Final[int]
    HKEY_LOCAL_MACHINE: Final[int]
    HKEY_USERS: Final[int]
    HKEY_PERFORMANCE_DATA: Final[int]
    HKEY_CURRENT_CONFIG: Final[int]
    HKEY_DYN_DATA: Final[int]

    KEY_ALL_ACCESS: Final = 983103
    KEY_WRITE: Final = 131078
    KEY_READ: Final = 131097
    KEY_EXECUTE: Final = 131097
    KEY_QUERY_VALUE: Final = 1
    KEY_SET_VALUE: Final = 2
    KEY_CREATE_SUB_KEY: Final = 4
    KEY_ENUMERATE_SUB_KEYS: Final = 8
    KEY_NOTIFY: Final = 16
    KEY_CREATE_LINK: Final = 32

    KEY_WOW64_64KEY: Final = 256
    KEY_WOW64_32KEY: Final = 512

    REG_BINARY: Final = 3
    REG_DWORD: Final = 4
    REG_DWORD_LITTLE_ENDIAN: Final = 4
    REG_DWORD_BIG_ENDIAN: Final = 5
    REG_EXPAND_SZ: Final = 2
    REG_LINK: Final = 6
    REG_MULTI_SZ: Final = 7
    REG_NONE: Final = 0
    REG_QWORD: Final = 11
    REG_QWORD_LITTLE_ENDIAN: Final = 11
    REG_RESOURCE_LIST: Final = 8
    REG_FULL_RESOURCE_DESCRIPTOR: Final = 9
    REG_RESOURCE_REQUIREMENTS_LIST: Final = 10
    REG_SZ: Final = 1

    REG_CREATED_NEW_KEY: Final = 1  # undocumented
    REG_LEGAL_CHANGE_FILTER: Final = 268435471  # undocumented
    REG_LEGAL_OPTION: Final = 31  # undocumented
    REG_NOTIFY_CHANGE_ATTRIBUTES: Final = 2  # undocumented
    REG_NOTIFY_CHANGE_LAST_SET: Final = 4  # undocumented
    REG_NOTIFY_CHANGE_NAME: Final = 1  # undocumented
    REG_NOTIFY_CHANGE_SECURITY: Final = 8  # undocumented
    REG_NO_LAZY_FLUSH: Final = 4  # undocumented
    REG_OPENED_EXISTING_KEY: Final = 2  # undocumented
    REG_OPTION_BACKUP_RESTORE: Final = 4  # undocumented
    REG_OPTION_CREATE_LINK: Final = 2  # undocumented
    REG_OPTION_NON_VOLATILE: Final = 0  # undocumented
    REG_OPTION_OPEN_LINK: Final = 8  # undocumented
    REG_OPTION_RESERVED: Final = 0  # undocumented
    REG_OPTION_VOLATILE: Final = 1  # undocumented
    REG_REFRESH_HIVE: Final = 2  # undocumented
    REG_WHOLE_HIVE_VOLATILE: Final = 1  # undocumented

    error = OSError

    # Though this class has a __name__ of PyHKEY, it's exposed as HKEYType for some reason
    @final
    class HKEYType:
        def __bool__(self) -> bool: ...
        def __int__(self) -> int: ...
        def __enter__(self) -> Self: ...
        def __exit__(
            self, exc_type: type[BaseException] | None, exc_value: BaseException | None, traceback: TracebackType | None, /
        ) -> bool | None: ...
        def Close(self) -> None: ...
        def Detach(self) -> int: ...
        def __hash__(self) -> int: ...
        @property
        def handle(self) -> int: ...



# This sample tests a case where a loop contains multiple conditional
# checks.

# pyright: strict

from __future__ import annotations


class ListNode:
    def __init__(self, val: int = 0, next: ListNode | None = None):
        self.val = val
        self.next = next


def has_cycle(head: ListNode | None) -> bool:
    fast_head = head
    while head and fast_head:
        fast_head = fast_head.next
        if fast_head:
            fast_head = fast_head.next
    return False



# This sample tests that member accesses to a TypedDict are properly
# handled even if one of the items in the TypedDict shadows the name
# of a TypedDict attribute.

from typing import TypedDict


class TD1(TypedDict):
    items: int


td1 = TD1(items=0)
td1.items()



# This sample tests the case where isinstance type narrowing is used
# with a protocol class that supports runtime checking.

# pyright: reportUnnecessaryIsInstance=true

from typing import Any, Iterable, Sized


def func1(v: Any) -> bool:
    if isinstance(v, Iterable):
        reveal_type(v, expected_text="Iterable[object]")
        if isinstance(v, Sized):
            reveal_type(v, expected_text="<subclass of Iterable[object] and Sized>")
            return True
    return False



# This sample tests the handling of a __getattr__ method that returns
# a callable. Such a method should not be bound.

from typing import Callable, TypeVar


class ClassA:
    def __init__(self):
        return

    def __getattr__(self, key: str) -> Callable[[str], str]:
        return lambda a: a


a = ClassA()

a.foo("hi")


T = TypeVar("T")


class MetaClass(type):
    def __getattr__(cls, key: str) -> Callable[[T], T]:
        return lambda x: x


class ClassB(metaclass=MetaClass):
    pass


v1 = ClassB.some_function(3)
reveal_type(v1, expected_text="int")

v2 = ClassB.some_function("hi")
reveal_type(v2, expected_text="str")



# This sample tests the case where the solver generates an unsolved
# unification variable that has been specialized into a conditional type.

from typing import Callable, Iterable, Self


class map[S]:
    def __new__[T](cls, func: Callable[[T], S], iter1: Iterable[T]) -> Self: ...


def func(a: list[int | None]):
    b = map(lambda x: x or 0, a)
    reveal_type(b, expected_text="map[int]")



# This sample file tests various aspects of type analysis for tuples.

no_args: tuple = ()
zero_length: tuple[()] = ()
all_ints1: tuple[int, ...] = ()
all_ints2: tuple[int, ...] = (1,)
all_ints3: tuple[int, ...] = (1, 3, 4)

all_ints1 = all_ints2
all_ints2 = all_ints3
all_ints3 = all_ints2

# This should generate an error.
bad_ellipsis1: tuple[...]

# This should generate an error.
bad_ellipsis2: tuple[int, int, ...]

# This should generate an error.
bad_ellipsis3: tuple[int, ..., int]

# This should generate an error.
bad_ellipsis4: tuple[*tuple[int], ...]

# This should generate an error.
bad_ellipsis5: tuple[*tuple[int, ...], ...]



# This sample validates that the type checker properly
# specializes a type for an unbound method (in this case,
# the "keys" method on "dict") based on the provided "self"
# argument.

v1: dict[str, str] = {}

# This should not result in an "Unknown", so no
# error should be generated.
result = dict.keys(v1)
reveal_type(result, expected_text="dict_keys[Unknown, Unknown]")



# This sample tests the type analyzer's handling of TypedDict classes.

from typing import TypedDict


class Movie1(TypedDict, total=False):
    name: str
    year: int


class Movie2(TypedDict, total=False):
    name: str
    year: int


class Movie3(TypedDict, total=True):
    name: str
    year: int


class Movie4(TypedDict, total=True):
    name: str
    year: int
    earnings: float


class Movie5(TypedDict, total=True):
    name: str
    year: float


movie1: Movie1 = Movie2(name="hello", year=1971)

# This should generate an error because
# items are required in Movie3 but not Movie2.
movie2: Movie2 = Movie3(name="hello", year=1971)

# This should generate an error because
# items are required in Movie3 but not Movie2.
movie3: Movie3 = Movie2(name="hello", year=1971)

# This should generate an error.
movie4: Movie4 = Movie3(name="hello", year=1971)

movie5: Movie3 = Movie4(name="hello", year=1971, earnings=23)


movie6 = Movie2(name="hello", year=1971)
movie6["name"] = "goodbye"

movie7 = {"name": "hello", "year": 1971}
movie7["name"] = "goodbye"

movie8: Movie2 = {"year": 1981, "name": "test"}
movie8["year"] = 1982

movie9 = Movie3(name="", year=1971)

# This should generate an error because "year" is mutable,
# so its type must match exactly.
movie10: Movie5 = movie9



# This sample tests the handling of Required and NotRequired
# (PEP 655) in TypedDict definitions.

# pyright: reportMissingModuleSource=false

from typing import Literal, TypedDict, Annotated
import typing_extensions as te
from typing_extensions import Required, NotRequired


class TD1(TypedDict, total=False):
    a: Annotated["te.Required[int]", ""]
    b: Annotated[te.NotRequired[str], ""]
    c: "te.Required[int | str]"
    d: te.Required[str | None]
    e: Required[Literal[1, 2, 3]]
    f: Required[None]
    g: Required[type[int]]


td1_1: TD1 = {"a": 3, "c": "hi", "d": None, "e": 3, "f": None, "g": int}

# This should generate an error because a is missing.
td1_2: TD1 = {"c": "hi", "d": None, "e": 3, "f": None, "g": int}

# This should generate an error because c is missing.
td1_3: TD1 = {"a": 3, "d": None, "e": 3, "f": None, "g": int}

# This should generate an error because d is missing.
td1_4: TD1 = {"a": 3, "c": "hi", "e": 3, "f": None, "g": int}

# This should generate an error because e is missing.
td1_5: TD1 = {"a": 3, "c": "hi", "d": None, "f": None, "g": int}

# This should generate an error because f is missing.
td1_6: TD1 = {"a": 3, "c": "hi", "d": None, "e": 3, "g": int}

# This should generate an error because g is missing.
td1_7: TD1 = {"a": 3, "c": "hi", "d": None, "e": 3, "f": None}


class TD2(TypedDict, total=True):
    a: Required[int]
    b: NotRequired[str]
    c: Required[int | str]
    d: NotRequired[str | None]
    e: NotRequired[Literal[1, 2, 3]]
    f: NotRequired[None]
    g: NotRequired[type[int]]


td2_1: TD2 = {"a": 3, "c": "hi", "d": None, "e": 3, "f": None, "g": int}

td2_2: TD2 = {"a": 3, "c": "hi"}

# This should generate an error because c is missing.
td2_3: TD2 = {"a": 3}



# This sample tests the handling of a bound TypeVar that is used
# in a Type[X] statement.

from typing import Callable, Generic, TypeVar


class Base: ...


T = TypeVar("T", bound=Base)


def register(state_name: str, state: type[T]): ...


def register_state(state_name: str) -> Callable[[type[T]], type[T]]:
    def decorator(state: type[T]) -> type[T]:
        register(state_name, state)
        return state

    return decorator


class F: ...


E = TypeVar("E", bound=F)


def coercer_method(value: E | str, enum: type[E]) -> E: ...


class C(Generic[E]):
    e_type: type[E]

    def coerce(self, e_type: type[E], value: E | str) -> E:
        return coercer_method(value, self.e_type)



# This sample tests the case where a lambda is passed to a constructor
# where both __new__ and __init__ methods are present and have different
# types.

# pyright: strict

from typing import Any, TypeVar, Callable

T = TypeVar("T")


def identity(val: T) -> T:
    return val


class ClassA:
    def __new__(cls, *args: Any, **kwargs: Any) -> "ClassA":
        return super().__new__(*args, **kwargs)

    def __init__(self, x: Callable[[float], float]) -> None:
        self.x = x


ClassA(lambda r: identity(r) + 1)



# This sample tests the case where an isinstance call uses
# a union of class types, some of which are tuples of other types
# and some of which are not.

from typing import TypeVar, Iterator

T1 = TypeVar("T1", bound="X")
T2 = TypeVar("T2", bound="X")


class X:
    element_list: list["X"]

    def return_iter(
        self, cls: type[T1] | tuple[type[T1], type[T2]]
    ) -> Iterator[T1 | T2]:
        for item in self.element_list:
            if isinstance(item, cls):
                yield item



# This sample tests the case where a union of two "bare" TypeVars are
# used in the annotation of a function parameter.

from typing import Any, TypeVar, Callable

S = TypeVar("S")
T = TypeVar("T")


def accepts_bool(b: bool) -> None: ...


def accepts_int(i: int) -> None: ...


def func1(x: S | T, l2: Callable[[S], Any], l3: Callable[[T], Any]) -> tuple[S, T]: ...


def func2(x: T | S, l2: Callable[[S], Any], l3: Callable[[T], Any]) -> tuple[S, T]: ...


x1 = func1(0, accepts_int, accepts_bool)
reveal_type(x1, expected_text="tuple[int, bool]")

x2 = func1(True, accepts_int, accepts_bool)
reveal_type(x2, expected_text="tuple[int, bool]")

x3 = func1(0, accepts_int, accepts_bool)
reveal_type(x3, expected_text="tuple[int, bool]")

x4 = func1(True, accepts_int, accepts_bool)
reveal_type(x4, expected_text="tuple[int, bool]")



# This file validates type narrowing that involve
# conditional binary expressions.

# pyright: reportOptionalMemberAccess=false

from random import random


class ClassA:
    def x(self):
        return


maybe = True

a = None if maybe else ClassA()
b = None if maybe else ClassA()

if not a or not b:
    a.x()
    b.x()
else:
    a.x()
    b.x()

if not (not a or not b):
    a.x()
    b.x()
else:
    a.x()
    b.x()

if not a and not b:
    # This should be flagged as an error
    a.x()
    # This should be flagged as an error
    b.x()
else:
    a.x()
    b.x()

if not (not a and not b):
    a.x()
    b.x()
else:
    # This should be flagged as an error
    a.x()
    # This should be flagged as an error
    b.x()

if a or b:
    a.x()
    b.x()
else:
    # This should be flagged as an error
    a.x()
    # This should be flagged as an error
    b.x()


def func1(a: str, b: str | bool) -> bool:
    x: str | bool = a and a in []
    reveal_type(x, expected_text="bool | Literal['']")

    if random() > 0.5:
        return (a and a in [""]) or True
    else:
        return x or True



# This sample tests the reportImplicitOverride diagnostic check
# (strict enforcement of PEP 698).

from typing import Any, Callable
from typing_extensions import override  # pyright: ignore[reportMissingModuleSource]


def evil_wrapper(func: Callable[..., Any], /):
    def wrapped(*args: Any, **kwargs: Any) -> Any:
        raise NotImplementedError

    return wrapped


class Base:
    @override
    def __init__(self):
        pass

    def method1(self):
        pass

    @property
    def prop_c(self) -> int:
        return 0

    def method2(self):
        pass


class Child(Base):
    def __init__(self):
        pass

    # This should generate an error if reportImplicitOverride is enabled.
    def method1(self):
        pass

    @property
    # This should generate an error if reportImplicitOverride is enabled.
    def prop_c(self) -> int:
        return 0

    @evil_wrapper
    def method2(self):
        pass



# This sample tests handling of tuples and tracking
# of specific types within a tuple.

from typing import Generic, TypeVar, Self

_T = TypeVar("_T")


class ClassA(tuple[int, str, int, _T]):
    def __new__(cls) -> Self: ...


objA = ClassA[complex]()

(a, b, c, d) = objA

aa1: int = a
bb1: str = b
cc1: int = c
dd1: complex = d

reveal_type(objA[0], expected_text="int")
reveal_type(objA[1], expected_text="str")
reveal_type(objA[2], expected_text="int")
reveal_type(objA[3], expected_text="complex")

# This should generate an error because the trailing
# comma turns the index value into a tuple.
e = objA[0,]

for aaa in objA:
    print(aaa)


class ClassB(tuple[_T, ...]):
    def __new__(cls) -> Self: ...


objB = ClassB[complex]()

(x, y, z) = objB

reveal_type(x, expected_text="complex")
reveal_type(y, expected_text="complex")
reveal_type(z, expected_text="complex")

xx2: complex = objB[0]
yy2: complex = objB[1]
zz2: complex = objB[2]


def func1(lst: list[str] | None) -> None:
    for item in lst or ():
        reveal_type(item, expected_text="str")


class X(Generic[_T]):
    def __init__(self):
        self._x: tuple[_T, ...] = ()



# This sample tests the tokenizer's ability to detect inconsistent
# use of tab and spaces in a way that's ambiguous.

def main(jsonIn):
    print("a")
	# This should generate an error because of inconsistent use of
	# tabs and spaces.
	print("b")




# This sample tests error detection for certain cases that
# are explicitly disallowed by PEP 572 for assignment expressions
# when used in context of a list comprehension.

pairs = []
stuff = []

# These should generate an error because assignment
# expressions aren't allowed within an iterator expression
# in a "for" clause of a list comprehension.
[x for x, y in (pairs2 := pairs) if x % 2 == 0]
[x for x, y in ([1, 2, 3, pairs2 := pairs]) if x % 2 == 0]
{x: y for x, y in (pairs2 := pairs) if x % 2 == 0}
{x for x, y in (pairs2 := pairs) if x % 2 == 0}
foo = (x for x, y in ([1, 2, 3, pairs2 := pairs]) if x % 2 == 0)

# This should generate an error because 'j' is used as a
# "for target" and the target of an assignment expression.
[[(j := j) for i in range(5)] for j in range(5)]
[i := 0 for i, j in stuff]
[i + 1 for i in (i := stuff)]

[False and (i := 0) for i, j in stuff]
[i for i, j in stuff if True or (j := 1)]

# These should generate an error because assignment
# expressions aren't allowed within an iterator expression
# in a "for" clause of a list comprehension.
[i + 1 for i in (j := stuff)]
[i + 1 for i in range(2) for j in (k := stuff)]
[i + 1 for i in [j for j in (k := stuff)]]
[i + 1 for i in (lambda: (j := stuff))()]


class Example:
    # This should generate an error because the containing
    # scope for the list comprehension is a class.
    [(j := i) for i in range(5)]

    x = ((y := 1), (z := 2))


Example.x
Example.y
Example.z

# This should generate an error because 'j' is used as a
# "for target" and the target of an assignment expression.
[i for i in [1, 2] if True or (j := 1) for j in range(10)]
